#--------------------------------------------------------------------------------------------------------------------
# this program obtains simulation results for the paper "The Sorting effects Method: Summarizing Effects Beyond Averages,"
# by V. Chernozhukov, I. Fernandez-Val, and Y. Luo
#--------------------------------------------------------------------------------------------------------------------
  
# Authors: V. Chernozhukov, I. Fernandez-Val, and Ye Luo

# Data source: IPUMS-CPS through Mert Demirer
# URL for data: https://cps.ipums.org/cps/

# The variables used in the analysis include:

# - lnw: log of hourly wage (annual earnings / annual hours)
# - female: female indicator
# - 6 married status indicators: widowed, divorced, separated, nevermarried, and married (omitted)
# - 5 education attainment indicators: lhs, hsg, cg, ad, and sc (omitted)
# - 4 region indicators: mw, so, we, and ne (omitted)
# - Quartic in potential experience (max[0, age - years of education - 7]): exp1, exp2 (divided by 100), exp3 (divided by 1000), exp4 (divided by 10000)
# - weight: March Supplement sampling weight
# - year: CPS year
# - Occupation categorical variable with 5 categories: manager, service, sales, construction, and production
# - Industry categorical variable with 12 categories: minery, construction, manufacture, retail, transport, information, finance, professional, education, 
#       leisure, services, and public

# This program calibrates a location model by ols and conducts a Monte Carlo simulation

# Uses weighted bootstrap with standard exponential weights for inference

# Updated on 05/21/2018

############

 
# Main Analysis;

rm(list=ls());

library(xtable); # library for nice tables
library(boot)  #imports the boot library
library(Hmisc); #library for weighted quantiles

# Functions;

# Fuction to estimate weighted distribution;

wcdf <- function(y, x, w)
{;
  Ff    <- weighted.mean((x <= y), w); 
  return(Ff);
};

# Function to generate bootstrap data sets;

data.rg <- function(data, mle)
{
  bdata <- data;
  n<-dim(data)[1];
  multipliers  <- rexp(n);
  bdata$weight <- data$weight * multipliers / sum(multipliers);
  return(bdata);
}

########## statistics to be computed in each bootstrap draw #####################################
boot.PE<- function(data, fmla1, us){
  n<-dim(data)[1]
  lm.fit.b <- lm(fmla1, data=data, weights = weight);
  ## design matrices for PE
  d0<- lm.fit.b$model;  
  d0[,2]<- rep(0,n);
  d1<- lm.fit.b$model;  
  d1[,2]<- rep(1,n);
  PE.lm   <- predict(lm.fit.b, newdata=d1) -  predict(lm.fit.b, newdata=d0);
  PET.lm  <- PE.lm[data$female==1];
  PET.weight <- data$weight[data$female==1];
  return(wtd.quantile(PET.lm, PET.weight, us));
}

# Calibration of DGP to the data by OLS;

options(warn=-1); # sets warnings off;
load('wage2015.rdata');
n<-dim(data)[1];

# create education factor;
data$educ <- factor(data$lhs+2*data$hsg+3*data$sc+4*data$cg+5*data$ad);

attach(data);
us    <- c(2:98)/100; # percentiles of interest for SPE
alpha  <- .1 #Significance level
beta   <- .1; #Tail quantile index for classication analysis

fmla1      <- lnw ~ female* (widowed + divorced + separated + nevermarried + (exp1+exp2+exp3+exp4)*(educ+occ2+ind2) + educ*(occ2+ind2) + mw + so + we); # with interactions
lm.fit     <- lm(fmla1, weights = weight);
xtable(lm.fit);   #report a nice table

## female = 0 desing matrix
d0<- lm.fit$model;  
d0[,2]<- rep(0,n);
## female = 1 design matrix
d1<- lm.fit$model;  
d1[,2]<- rep(1,n);

PE.lm   <- predict(lm.fit, newdata=d1) -  predict(lm.fit, newdata=d0);
PET     <- matrix(PE.lm[which(female==1)], ncol = 1, byrow = FALSE);  
PET.w   <- matrix(weight[female==1], ncol = 1, nrow = nrow(PET), byrow = FALSE);
SPET0    <- wtd.quantile(PET, PET.w, us);   # these are estimated SPEs

plnw  <- predict(lm.fit);
sigma <- sqrt(wtd.var(lm.fit$res, weight));

# Simulation;

S <- 500; # number of simulations

SPET <- matrix(0, nrow = length(us), ncol = S);
SPET.bc <- matrix(0, nrow = length(us), ncol = S);

uSPET <- matrix(0, nrow = length(us), ncol = S);
uSPET.bc <- matrix(0, nrow = length(us), ncol = S);

lSPET <- matrix(0, nrow = length(us), ncol = S);
lSPET.bc <- matrix(0, nrow = length(us), ncol = S);

set.seed(1);

for (s in 1:S) {
  
  detach(data);
  data$lnw <- plnw + sigma * rnorm(n);
  attach(data);

  lm.fit     <- lm(fmla1, weights = weight);

  PE.lm   <- predict(lm.fit, newdata=d1) -  predict(lm.fit, newdata=d0);
  PET     <- matrix(PE.lm[which(female==1)], ncol = 1, byrow = FALSE);  
  PET.w   <- matrix(weight[female==1], ncol = 1, nrow = nrow(PET), byrow = FALSE);
  SPET.lm <- wtd.quantile(PET, PET.w, us);
  SPET[, s] <- SPET.lm;   # these are estimated SPEs 
  
  # Inference by bootstrap;

  result.boot.PE.lm <- boot(data = data, statistic=boot.PE, sim = "parametric", ran.gen = data.rg, mle = 0, fmla1 = fmla1, us = us, parallel="multicore", ncpus = 20, R=200);

  est.SPET   <- SPET.lm;   # these are estimated SPETs
  draws.SPET <- result.boot.PE.lm$t;  #these are bs draws
  delta      <- draws.SPET-matrix(est.SPET, nrow = nrow(draws.SPET), ncol = ncol(draws.SPET), byrow=TRUE);
  variance   <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
  zs         <- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.SPET), ncol = ncol(draws.SPET), byrow=TRUE)), 1, max);  #max abs t-stat
  crt        <- quantile(zs, 1-alpha);  #critical value

  uSPET[, s] <-  sort(est.SPET + crt*sqrt(variance));  #upper conf band
  lSPET[, s] <-  sort(est.SPET - crt*sqrt(variance));  #lower conf band

  # Bootstrap bias correction;

  SPET.bc[, s]   <- sort(2*est.SPET - apply(draws.SPET,2,mean));
  uSPET.bc[, s]  <- sort(2*est.SPET - apply(draws.SPET,2,mean) + crt*sqrt(variance));  #upper conf band
  lSPET.bc[, s]  <- sort(2*est.SPET - apply(draws.SPET,2,mean) - crt*sqrt(variance));  #lower conf band

}

# Figure of results;


pdf("Gender-SPET-lm-full2-mc-bias-percent.pdf", pointsize=15,width=8.0,height=8.0)
plot(us, apply(SPET, 1, mean) - SPET0, type="l",col=1, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
          main="Bias",
     # sub="Mean model", 
     ylim=c(-.1,.1),
     lty = 2)

lines(us, apply(SPET.bc, 1, mean) - SPET0, lwd=2, col = 2, lty = 1 )
abline(h=0)
legend( x="topleft", 
        legend=c("Empirical SPET","Bias Corrected"), 
        col=c(1,2), lwd=c(2,2), lty=c(2, 1),  bty = 'n' )

dev.off();

pdf("Gender-SPET-lm-full2-mc-rmse.pdf", pointsize=15,width=8.0,height=8.0)
plot(us, sqrt((apply(SPET, 1, mean) - SPET0)^2 + apply(SPET, 1, var)), type="l",col=1, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
          main="RMSE",
     # sub="Mean model", 
     ylim=c(0,.12),
     lty = 2)

lines(us, sqrt((apply(SPET.bc, 1, mean) - SPET0)^2 + apply(SPET.bc, 1, var)), lwd=2, col = 2, lty = 1 )
abline(h=0)
legend( x="topleft", 
        legend=c("Empirical SPET","Bias Corrected"), 
        col=c(1,2), lwd=c(2,2), lty=c(2, 1),  bty = 'n' )

dev.off();

pdf("Gender-SPET-lm-full2-mc-sd.pdf", pointsize=15,width=8.0,height=8.0)
plot(us, apply(SPET, 1, sd), type="l",col=1, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
          main="Std. Dev.",
     # sub="Mean model", 
     ylim=c(0,.06),
     lty = 2)

lines(us, apply(SPET.bc, 1, sd), lwd=2, col = 2, lty = 1 )
abline(h=0)
legend( x="topleft", 
        legend=c("Empirical SPET","Bias Corrected"), 
        col=c(1,2), lwd=c(2,2), lty=c(2, 1),  bty = 'n' )

dev.off();


# Table of results;

table.cov<- matrix(0,2,1);
colnames(table.cov)<- c('Coverage');
rownames(table.cov)<- c('Uncorrected','BC-Boot');
table.cov[1,]<- mean(apply((matrix(SPET0, nrow = length(us), ncol = S) >= lSPET) & (matrix(SPET0, nrow = length(us), ncol = S) <= uSPET), 2, min));
table.cov[2,]<- mean(apply((matrix(SPET0, nrow = length(us), ncol = S) >= lSPET.bc) & (matrix(SPET0, nrow = length(us), ncol = S) <= uSPET.bc), 2, min));

xtable(t(table.cov), caption = "Coverage of 90% Confidence Bands", digits=2, add.to.row = comment, hline.after = c(-1, 0));

save.image(file="Gender-SPET-lm-full-S500.RData");



