#--------------------------------------------------------------------------------------------------------------------
# this program obtains Empirical results for the paper "The Sorted Effects Method: Summarizing Effects Beyond Averages,"
# by V. Chernozhukov, I. Fernandez-Val, and Y. Luo
#--------------------------------------------------------------------------------------------------------------------

# Authors: V. Chernozhukov, I. Fernandez-Val, and Ye Luo

# Data source: IPUMS-CPS through Mert Demirer
# URL for data: https://cps.ipums.org/cps/

# The variables used in the analysis include:

# - lnw: log of hourly wage (annual earnings / annual hours)
# - female: female indicator
# - 6 married status indicators: widowed, divorced, separated, nevermarried, and married (omitted)
# - 5 education attainment indicators: lhs, hsg, cg, ad, and sc (omitted)
# - 4 region indicators: mw, so, we, and ne (omitted)
# - Quartic in potential experience (max[0, age - years of education - 7]): exp1, exp2 (divided by 100), exp3 (divided by 1000), exp4 (divided by 10000)
# - weight: March Supplement sampling weight
# - year: CPS year
# - Occupation categorical variable with 5 categories: manager, service, sales, construction, and production
# - Industry categorical variable with 12 categories: minery, construction, manufacture, retail, transport, information, finance, professional, education, 
#       leisure, services, and public

# This program does the ca for subpopulations using the additive model for the conditional expectation

# Uses weighted bootstrap with standard exponential weights for inference

# Specification that excludes occupation and industry indicators and interactions

# Updated on 10/09/2017

############


# Main Analysis;

library(xtable); # library for nice tables
library(boot)  #imports the boot library
library(Hmisc); #library for weighted quantiles

options(warn=-1); # sets warnings off;
load('wage2015.rdata');

# create occupation indicators;
data$occ.manager <- (data$occ2==1);
data$occ.service <- (data$occ2==2);
data$occ.sales   <- (data$occ2==3);
data$occ.construction <- (data$occ2==4);
data$occ.production <- (data$occ2==5);

data$occ2 <- factor(data$occ2, labels=c('man','ser','sal','con','prod'));

# create industry indicators;
data$ind.minery <- (data$ind2==2);
data$ind.construction <- (data$ind2==3);
data$ind.manufacture <- (data$ind2==4);
data$ind.retail <- (data$ind2==5);
data$ind.transport <- (data$ind2==6);
data$ind.information <- (data$ind2==7);
data$ind.finance <- (data$ind2==8);
data$ind.professional <- (data$ind2==9);
data$ind.education <- (data$ind2==10);
data$ind.leisure <- (data$ind2==11);
data$ind.services <- (data$ind2==12);
data$ind.public <- (data$ind2==13);

data$ind2 <- factor(data$ind2, labels=c('min','cons','man','ret','tra','inf','fin','pro','edu','lei','ser','pub'));

# create education factor;
data$educ <- factor(data$lhs+2*data$hsg+3*data$sc+4*data$cg+5*data$ad, labels = c('lhs','hsg','sc','cg','ad'));

# create marital status factor;
data$ms <- factor(data$married+2*data$widowed+3*data$separated+4*data$divorced+5*data$nevermarried, labels =c('mar','wid','sep','div','nev'));

# create region factor;
data$region <- factor(data$ne+2*data$we +3*data$mw+4*data$so, labels =c('northeast','west','midwest','south'));

attach(data);
n<-dim(data)[1];
us    <- c(2:98)/100; # percentiles of interest for SPE
alpha  <- .1 #Significance level
beta   <- .1; #Tail quantile index for classication analysis
R <- 500;

########## Descriptive statistics

vars <- c("lnw","female","married","widowed","separated","divorced","nevermarried","lhs","hsg","sc","cg","ad","ne","mw","so","we",
          "exp1","occ.manager","occ.service","occ.sales","occ.construction","occ.production", "ind.minery", "ind.construction", 
          "ind.manufacture", "ind.retail", "ind.transport", "ind.information", "ind.finance", "ind.professional", 
          "ind.education", "ind.leisure", "ind.services", "ind.public");
options(digits=2);
dstats <- cbind(sapply(data[,vars], weighted.mean, weight), sapply(data[,vars], sd), apply(data[female==1,vars], 2, weighted.mean, weight[female==1]), apply(data[female==0,vars], 2, weighted.mean, weight[female==0]));

xtable(dstats);


###############################################################################################
# FULL SPECIFICATION
###############################################################################################

# Part 1: Unconditional effects;

fmla1      <- lnw ~ female* (widowed + divorced + separated + nevermarried + exp1+exp2+exp3+exp4+educ+occ2+ind2 + mw + so + we); # specification without three-way interactions
lm.fit     <- lm(fmla1, weights = weight); 

## female = 0 desing matrix
d0<- lm.fit$model;  
d0[,2]<- rep(0,n);
## female = 1 design matrix
d1<- lm.fit$model;  
d1[,2]<- rep(1,n);

PE.lm       <- predict(lm.fit, newdata=d1) -  predict(lm.fit, newdata=d0);
PET.lm      <- PE.lm[female==1];
PET.weight  <- weight[female==1];

high.effect<- wtd.quantile(PET.lm, PET.weight, 1-beta);
low.effect <- wtd.quantile(PET.lm, PET.weight, beta);
print(high.effect); print(low.effect);



# Inference by bootstrap;

# Function to generate bootstrap data sets;

data.rg <- function(data, mle)
{
  bdata <- data;
  n<-dim(data)[1];
  multipliers  <- rexp(n);
  bdata$weight <- data$weight * multipliers / sum(multipliers);
  return(bdata);
}


########## statistics to be computed in each bootstrap draw #####################################
boot.SE<- function(data, fmla1, beta){
  n<-dim(data)[1]
  lm.fit.b <- lm(fmla1, data=data, weights = weight);
  ## design matrices for PE
  d0<- lm.fit.b$model;  
  d0[,2]<- rep(0,n);
  d1<- lm.fit.b$model;  
  d1[,2]<- rep(1,n);
  PE.lm   <- predict(lm.fit.b, newdata=d1) -  predict(lm.fit.b, newdata=d0);
  PET.lm  <- PE.lm[data$female==1];
  PET.weight <- data$weight[data$female==1];
  high.effect<- wtd.quantile(PET.lm, PET.weight, 1-beta);
  low.effect <- wtd.quantile(PET.lm, PET.weight, beta);
  return(c(high.effect,low.effect,PET.lm));
}

set.seed(88);
result.boot.SE.lm <- boot(data = data, statistic=boot.SE, sim = "parametric", ran.gen = data.rg, mle = 0, fmla1 = fmla1, beta = beta, parallel="multicore", ncpus = 20, R=R);

# CLASSIFICATION ANALYSIS FOR SUBPOPULATIONS;

is.he0      <- rank(abs(PET.lm - high.effect)) == min(rank(abs(PET.lm - high.effect)));
draws.PE.he <- result.boot.SE.lm$t[,-c(1,2)] - matrix(result.boot.SE.lm$t[,1], nrow = R, ncol = length(PET.lm)) - matrix(PET.lm - high.effect, nrow = R, ncol = length(PET.lm), byrow = TRUE) ;
bse.PE.he   <- (apply(draws.PE.he, 2, quantile, .75, na.rm=TRUE) - apply(draws.PE.he, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
bm.PE.he   <-  apply(draws.PE.he, 2, mean); # bias estimator
zs.PE.he    <- apply(-draws.PE.he[,is.he0] /matrix(bse.PE.he[is.he0], nrow = R, ncol = length(bse.PE.he[is.he0]), byrow=TRUE), 1, max, na.rm = TRUE);  #max abs t-stat
crt.PE.he   <- quantile(zs.PE.he, 1 - alpha);  #critical value
is.he       <- -(PET.lm - high.effect)/bse.PE.he <= crt.PE.he; 

is.le0      <- rank(abs(PET.lm - low.effect)) == min(rank(abs(PET.lm - low.effect)));
draws.PE.le <- result.boot.SE.lm$t[,-c(1,2)] - matrix(result.boot.SE.lm$t[,2], nrow = R, ncol = length(PET.lm)) - matrix(PET.lm - low.effect, nrow = R, ncol = length(PET.lm), byrow = TRUE) ;
bse.PE.le   <- (apply(draws.PE.le, 2, quantile, .75, na.rm=TRUE) - apply(draws.PE.le, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
bm.PE.le   <-  apply(draws.PE.le, 2, mean); # bias estimator
zs.PE.le    <- apply(draws.PE.le[,is.le0] /matrix(bse.PE.le[is.le0], nrow = R, ncol = length(bse.PE.le[is.le0]), byrow=TRUE), 1, max, na.rm = TRUE);  #max abs t-stat
crt.PE.le   <- quantile(zs.PE.le, 1-alpha);  #critical value
is.le       <- (PET.lm - low.effect)/bse.PE.le <= crt.PE.le; 


fdata <- data[female==1,];

#######################################
# FIGURES OF RESULTS;
#######################################

pdf("Gender-CSPET-lm-full-ca-by_exp-nointer.pdf", pointsize=15,width=12.0,height=12.0);

plot(fdata$exp1, fdata$lnw, type="n",col=4, pch = 20, xlab="Experience", lwd=2,
     ylab="Log Wage", 
     xlim=range(fdata$exp1));
points(fdata$exp1[is.he & !is.le], fdata$lnw[is.he & !is.le], col = 'lightblue1', pch = 1, lwd=2)
points(fdata$exp1[is.le & !is.he], fdata$lnw[is.le & !is.he], col = 4, pch = 20, lwd=2)
legend('topleft', c('10% Most','10% Least'), col = c(4,'lightblue1'), pch = c(20, 1), horiz = F, bty = 'n');

plot(fdata$exp1, fdata$ms, type="n",col=4, pch = 20, xlab="Experience", lwd=2,
     ylab="Marital Status", 
     xlim=range(fdata$exp1), 
     yaxt = 'n');
axis(side=2, at=c(1:5),labels=levels(fdata$ms));
points(fdata$exp1[is.he & !is.le], fdata$ms[is.he & !is.le], col = 'lightblue1', pch = 1, lwd=2)
points(fdata$exp1[is.le & !is.he], fdata$ms[is.le & !is.he], col = 4, pch = 20, lwd=1)
legend('topleft', c('10% Most','10% Least'), col = c(4,'lightblue1'), pch = c(20, 1), horiz = F, bty = 'n');

plot(fdata$exp1, fdata$educ, type="n",col=4, pch = 20, xlab="Experience", lwd=2,
     ylab="Education", 
     xlim=range(fdata$exp1), 
     yaxt = 'n');
axis(side=2, at=c(1:5),labels=levels(fdata$educ));
points(fdata$exp1[is.he & !is.le], fdata$educ[is.he & !is.le], col = 'lightblue1', pch = 1, lwd=2)
points(fdata$exp1[is.le & !is.he], fdata$educ[is.le & !is.he], col = 4, pch = 20, lwd=1)
legend('topleft', c('10% Most','10% Least'), col = c(4,'lightblue1'), pch = c(20, 1), horiz = F, bty = 'n');

plot(fdata$exp1, fdata$region, type="n",col=4, pch = 20, xlab="Experience", lwd=2,
     ylab="Region",
     xlim=range(fdata$exp1),
     yaxt = 'n');
axis(side=2, at=c(1:4),labels=levels(fdata$region));
points(fdata$exp1[is.he & !is.le], fdata$region[is.he & !is.le], col = 'lightblue1', pch = 1, lwd=2)
points(fdata$exp1[is.le & !is.he], fdata$region[is.le & !is.he], col = 4, pch = 20, lwd=1)
legend('topleft', c('10% Most','10% Least'), col = c(4,'lightblue1'), pch = c(20, 1), horiz = F, bty = 'n');

plot(fdata$exp1, fdata$occ2, type="n",col=4, pch = 20, xlab="Experience", lwd=2,
     ylab="Occupation", 
     xlim=range(fdata$exp1), 
     yaxt = 'n');
axis(side=2, at=c(1:5),labels=levels(fdata$occ2));
points(fdata$exp1[is.he & !is.le], fdata$occ2[is.he & !is.le], col = 'lightblue1', pch = 1, lwd=2)
points(fdata$exp1[is.le & !is.he], fdata$occ2[is.le & !is.he], col = 4, pch = 20, lwd=1)
legend('topleft', c('10% Most','10% Least'), col = c(4,'lightblue1'), pch = c(20, 1), horiz = F, bty = 'n');

plot(fdata$exp1, fdata$ind2, type="n",col=4, pch = 20, xlab="Experience", lwd=2,
     ylab="Industry",
     xlim=range(fdata$exp1),
     yaxt = 'n');
axis(side=2, at=c(1:12),labels=levels(fdata$ind2));
points(fdata$exp1[is.he & !is.le], fdata$ind2[is.he & !is.le], col = 'lightblue1', pch = 1, lwd=2)
points(fdata$exp1[is.le & !is.he], fdata$ind2[is.le & !is.he], col = 4, pch = 20, lwd=1)
legend('topleft', c('10% Most','10% Least'), col = c(4,'lightblue1'), pch = c(20, 1), horiz = F, bty = 'n')



dev.off();



save.image(file="Gender-CSPET-lm-ca-nointer.RData");
rm(result.boot.PE.lm);
detach(data);
