#--------------------------------------------------------------------------------------------------------------------
# this program obtains Empirical results for the paper "The Sorting Effects Method: Summarizing Effects Beyond Averages,"
# by V. Chernozhukov, I. Fernandez-Val, and Y. Luo
#--------------------------------------------------------------------------------------------------------------------

#  The results are based on the data used in Alicia H. Munnell, Geoffrey M. B. Tootell, 
# Lynn E. Browne and James McEneaney (1996, AER) to measure the effect of being back on the
# probability of mortgage denial.  

#  Authors:  V. Chernozhukov, I. Fernandez-Val, and Y. Luo
#  RA: Vira Semenova and Mert Demirer
#  Credit to: A. Timoshenko (who fixed an error in the code).
#
# Data source: Boston HMDA Data Set (Stock and Watson version)
# URL for data: http://wps.aw.com/aw_stock_ie_3/178/45691/11696965.cw/index.html/

# Description of the data: the sample selection and variable contruction follow

# Alicia H. Munnell, Geoffrey M. B. Tootell, Lynn E. Browne and James McEneaney (1996) 
# "Mortgage Lending in Boston: Interpreting HMDA Data,"  The American Economic Review Vol. 86, No. 1 (Mar., 1996), pp. 25-53 

# Sample selection: single-residences (excluding data on multifamily homes), 2,380 observation

# The variables in the data set include:

# deny:  = 1 if mortgage application denied
# p_irat: montly debt to income ratio
# black: = 1 if applicant is black
# hse_inc: montly housing expenses to income ratio
# loan_val: loan to assessed property value ratio (not used in analysis)
# ccred: consumer credit score 
#        = 1 if no "slow" payments or delinquencies
#        = 2 if one or two "slow" payments or delinquencies
#        = 3 if more than two "slow" payments or delinquencies
#        = 4 if insufficient credit history for determination
#        = 5 if delinquent credit history with payments 60 days overdue
#        = 6 if delinquent credit history with payments 90 days overdue
# mcred: mortgage credit score
#        = 1 if no late mortgage payments
#        = 2 if no mortgage payment history
#        = 3 if one or two late mortgage payments
#        = 4 if more than two late mortgage payments
# pubrec: = 1 if any public record of credit problems (bankruptcy, charge-offs, collection actions)
# denpmi: = 1 if applicant applied for mortgage insurance and was denied
# selfemp: = 1 if self-employed
# single: = 1 if single
# hischl: = 1 if high school graduated
# probunmp: 1989 Massachusetts unemployment rate in the applicant's industry (not used in analysis)
# condo: = 1 if unit is a condominium (not used in analysis)
# ltv_med: = 1 if medium loan to property value ratio [.80, .95]
# ltv_high: = 1 if high loan to property value ratio > .95
#################################################################################################


# Updated on 04/25/2017

# Add bootstrap bias corrections and std errors for classification analysis


##################################################################################################


###  read-in Mortgage data

library(foreign);
library(xtable);

data.In<- read.dta(mortgage.dta);
n1<- dim(data.In)[1];

########## Descriptive statistics

options(digits=2);
dstats <- cbind(sapply(data.In, mean), apply(data.In[data.In$black==1,], 2, mean), apply(data.In[data.In$black==0,], 2, mean));
xtable(dstats);


################################# Part 2. Predicted Effects of Black ##################

###############################################################################################
# logistic model
###############################################################################################

## Estimate model with entire sample
attach(data.In)
n <- n1;
fmla1 =  deny ~ black + p_irat + hse_inc + ccred + mcred  + pubrec + ltv_med + ltv_high + denpmi + selfemp + single + hischl                  
fit.lgt.1=glm(fmla1, family=binomial(link="logit"))

## black=0 design matrix
d0<- fit.lgt.1$model;  d0[,2]<- rep(0,n)
## black = 1 design matrix
d1<- fit.lgt.1$model;  d1[,2]<- rep(1,n)

effect<- predict(fit.lgt.1, newdata=d1, type="response") -  predict(fit.lgt.1, newdata=d0, type="response");
mean(effect); sqrt(var(effect));
min(effect); max(effect);

mean(effect[black==1]); sqrt(var(effect[black==1]));
min(effect[black==1]); max(effect[black==1]);

################################ Part 3. BOOTSTRAP THE APE and SPE Logit Case ###########################

library(boot)  #imports the boot library

alpha  <- .10 #Significance level
us    <- c(2:98)/100; # percentiles of interest for SPE


########## statistic to be computed in each bootstrap draw #####################################

boot.PE<- function(data,indices){
data.b<- data[indices,];  
fmla1 =  deny ~ black + p_irat + hse_inc + ccred + mcred  + pubrec + ltv_med + ltv_high + denpmi + selfemp + single + hischl                  
#basic logistic model, linear index
fit.lgt.1=glm(fmla1, data=data.b, family=binomial(link="logit"))
## black=0 design matrix
n<-dim(data.b)[1]
d0<- fit.lgt.1$model;  d0[,2]<- rep(0,n)
## black = 1 design matrix
d1<- fit.lgt.1$model;  d1[,2]<- rep(1,n)
effect<- c(mean(predict(fit.lgt.1, newdata=d1, type="response")-predict(fit.lgt.1, newdata=d0, type="response")), 
           quantile( predict(fit.lgt.1, newdata=d1, type="response")-predict(fit.lgt.1, newdata=d0, type="response"), us) );
return(effect)
}

################### application of bootstrap ##################################################

set.seed(1)

# bootstrap using parallelization

result.boot.PE<- boot(data=data.In, statistic=boot.PE, parallel="multicore", ncpus = 24, R=500)

################## estimate SPE and processes BS results #######################################

est.SPE<- quantile(effect, us);   # these are estimated SPEs

draws.SPE<- result.boot.PE$t[, -1];  #these are bs draws

# centered/scaled draws
delta     <- draws.SPE-matrix(est.SPE, nrow = nrow(draws.SPE), ncol = ncol(draws.SPE), byrow=TRUE)
variance  <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs        <- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.SPE), ncol = ncol(draws.SPE), byrow=TRUE)), 1, max)  #max abs t-stat
crt       <- quantile(zs, 1-alpha)  #critical value

ubound.SPE<-  sort(est.SPE + crt*sqrt(variance));  #upper conf band
lbound.SPE<-  sort(est.SPE - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.SPE.bc    <- sort(2*est.SPE - apply(draws.SPE,2,mean));
ubound.SPE.bc <-  sort(2*est.SPE - apply(draws.SPE,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.SPE.bc <-  sort(2*est.SPE - apply(draws.SPE,2,mean) - crt*sqrt(variance));  #lower conf band


################# Part 3. Estimate the APE and process BS results ########################

est.APE<- mean(effect);   # estimated APE

draws.APE  <- result.boot.PE$t[, 1];    #bs draws of APEs
variance   <- ((quantile(draws.APE, .75, na.rm=TRUE) - quantile(draws.APE, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs<- abs(draws.APE-est.APE)/sqrt(variance);   # abs t-stat
crt2<- quantile(mzs, 1-alpha);   #critical value

ubound.APE <-  est.APE+ crt2*sqrt(variance);  #upper conf bound
lbound.APE <- est.APE - crt2*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.APE.bc    <- 2*est.APE - mean(draws.APE);
ubound.APE.bc <- est.APE.bc + crt2*sqrt(variance);  #upper conf bound
lbound.APE.bc <- est.APE.bc - crt2*sqrt(variance);  #lower conf bound

# Plot of SPE and APE;

  pdf("mortgage-SPE.pdf", pointsize=15,width=8,height=8)
  plot(us, est.SPE, type="l",col=4, xlab="Percentile index", lwd=2,
       ylab="Change in probability", 
       sub="Logit model", 
       ylim=c(min(lbound.SPE), max(ubound.SPE)))
  
  polygon(c(us,rev(us)),c(ubound.SPE,rev(lbound.SPE)), density=60, border=F, 
          col='light blue', lty =1, lwd = 1);
  lines(us, est.SPE, lwd=2, col=1 )
  lines(us, rep(est.APE,length(us)), col=1 ,lwd=2)
  points( c(min(us),.2,.4,.6,.8,max(us)), rep(est.APE,6),  col=1, pch=15 )
  lines(us, rep(ubound.APE,length(us)), col=1 , lty =2, lwd=2)
  lines(us, rep(lbound.APE,length(us)), col=1 , lty =2, lwd=2)
  legend( x="topleft", 
          legend=c("SPE","APE","90% CB(SPE)", "90% CB(APE)"), 
          col=c(1,1,"light blue",1), lwd=c(1, 1, 5, 1), lty=c(1, 1, 1, 2), pch=c(NA, 15, NA, NA), pt.cex = c(2, 1), bty = 'n' )
  dev.off();
  
  pdf("mortgage-SPE-bc.pdf", pointsize=15,width=8,height=8)
  plot(us, est.SPE.bc, type="l",col=4, xlab="Percentile index", lwd=2,
       ylab="Change in probability", 
       #       main="Effect of Black on Probability of Mortgage Denial",
       sub="Logit model", 
       ylim=c(min(lbound.SPE), max(ubound.SPE)))
  
  polygon(c(us,rev(us)),c(ubound.SPE.bc,rev(lbound.SPE.bc)), density=60, border=F, 
          col='light blue', lty =1, lwd = 1);
  lines(us, est.SPE.bc, lwd=2, col=1 )
  lines(us, rep(est.APE.bc,length(us)), col=1 ,lwd=2)
  points( c(min(us),.2,.4,.6,.8,max(us)), rep(est.APE.bc,6),  col=1, pch=15 )
  lines(us, rep(ubound.APE.bc,length(us)), col=1 , lty =2, lwd=2)
  lines(us, rep(lbound.APE.bc,length(us)), col=1 , lty =2, lwd=2)
  legend( x="topleft", 
          legend=c("SPE","APE","90% CB(SPE)", "90% CB(APE)"), 
          col=c(1,1,"light blue",1), lwd=c(1, 1, 5, 1), lty=c(1, 1, 1, 2), pch=c(NA, 15, NA, NA), pt.cex = c(2, 1), bty = 'n' )
  dev.off();
  

#################### Part 4: Whose probabilities are most affected? ################################

beta <- .10; # tail quantile for classification analysis
  
high.effect<- quantile(effect, 1-beta)
low.effect<- quantile(effect, beta)
print(high.effect); print(low.effect);

colnames<- names(d1)
high.affected<- data.In[effect>=high.effect,colnames]
low.affected<- data.In[effect<=low.effect,colnames]

########## statistics to be computed in each bootstrap draw #####################################
boot.SE<- function(data, indices, fmla1, us, vars, beta){
  n<-dim(data)[1];
  data.b<- data[indices,];  
  #fmla1 =  deny ~ black + p_irat + hse_inc + ccred + mcred  + pubrec + ltv_med + ltv_high + denpmi + selfemp + single + hischl                  
  #basic logistic model, linear index
  fit.lgt.1=glm(fmla1, data=data.b, family=binomial(link="logit"))
  ## black=0 design matrix
  d0<- fit.lgt.1$model;  d0[,2]<- rep(0,n)
  ## black = 1 design matrix
  d1<- fit.lgt.1$model;  d1[,2]<- rep(1,n)
  PE   <- predict(fit.lgt.1, newdata=d1, type="response")-predict(fit.lgt.1, newdata=d0, type="response");
  mean.ha <- apply(data.b[PE>=quantile(PE, 1-beta), vars], 2, mean);
  mean.la <- apply(data.b[PE<=quantile(PE, beta), vars], 2, mean);
  sd.ha <- apply(data.b[PE>=quantile(PE, 1-beta), vars], 2, sd);
  sd.la <- apply(data.b[PE<=quantile(PE, beta), vars], 2, sd);
  return(c(mean.ha, mean.la, sd.ha, sd.la));
}

set.seed(1);
result.boot.SE <- boot(data=data.In, statistic=boot.SE, fmla1 = fmla1, us = us, vars = colnames, beta = beta, parallel="multicore", ncpus = 24, R=200)

# Standard Erros for Means in Classification Analysis;

vars     <- colnames;
draws    <- result.boot.SE$t[,1:(2*length(vars))];  #these are bs draws for the mean
bse      <- (apply(draws, 2, quantile, .75, na.rm=TRUE) - apply(draws, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust std dev estimator
draws.sd <- result.boot.SE$t[,2*length(vars) + 1:(2*length(vars))];  #these are bs draws for the sds
bse.sd   <- (apply(draws.sd, 2, quantile, .75, na.rm=TRUE) - apply(draws.sd, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator


# Table for the mean and std dev including estimates and std errors

table.full.who<- matrix(0,8,length(vars));
colnames(table.full.who)<- vars;
table.full.who[1,]<- apply(high.affected, 2, mean);
table.full.who[2,]<- bse[c(1:length(vars))];
table.full.who[3,]<- apply(high.affected, 2, sd);
table.full.who[4,]<- bse.sd[c(1:length(vars))];
table.full.who[5,]<- apply(low.affected, 2, mean);
table.full.who[6,]<- bse[-c(1:length(vars))];
table.full.who[7,]<- apply(low.affected, 2, sd);
table.full.who[8,]<- bse.sd[-c(1:length(vars))];

xtable(t(table.full.who), digits=2);

xtable(t(table.full.who[-c(3,4,7,8),]), digits=2)

save.image(file="mortgage.RData");

rm(draws.sd, bse.sd);

