#--------------------------------------------------------------------------------------------------------------------
# this program obtains Empirical results for the paper "The Sorted Effects Method: Summarizing Effects Beyond Averages,"
# by V. Chernozhukov, I. Fernandez-Val, and Y. Luo
#--------------------------------------------------------------------------------------------------------------------
   
# Authors: V. Chernozhukov, I. Fernandez-Val, and Ye Luo

# Data source: IPUMS-CPS through Mert Demirer
# URL for data: https://cps.ipums.org/cps/

# The variables used in the analysis include:

# - lnw: log of hourly wage (annual earnings / annual hours)
# - female: female indicator
# - 6 married status indicators: widowed, divorced, separated, nevermarried, and married (omitted)
# - 6 education attainment indicators: hsd08, hsd911, hsg, cg, ad, and sc (omitted)
# - 4 region indicators: mw, so, we, and ne (omitted)
# - Quartic in potential experience (max[0, age - years of education - 7]): exp1, exp2 (divided by 100), exp3 (divided by 1000), exp4 (divided by 10000)
# - weight: March Supplement sampling weight
# - year: CPS year
# - Occupation categorical variable with 5 categories: manager, service, sales, construction, and production
# - Industry categorical variable with 12 categories: minery, construction, manufacture, retail, transport, information, finance, professional, education, 
#       leisure, services, and public

# This program estimates the nonadditive model for the conditional quantiles by qr

# Uses weighted bootstrap with standard exponential weights for inference

# Updated on 05/24/2018

############


# Main Analysis;

rm(list=ls());

source('rq-nonstop.R'); # loads auxiliary functions for rq

library(xtable); # library for nice tables
library(boot)  #imports the boot library
library(Hmisc); #library for weighted quantiles
library(quantreg); #library for quantile regression

options(warn=-1); # sets warnings off;
load('wage2015.rdata'); #loads the data

# create occupation indicators;
data$occ.manager <- 1*(data$occ2==1);
data$occ.service <- 1*(data$occ2==2);
data$occ.sales   <- 1*(data$occ2==3);
data$occ.construction <- 1*(data$occ2==4);
data$occ.production <- 1*(data$occ2==5);

# create industry indicators;
data$ind.minery <- 1*(data$ind2==2);
data$ind.construction <- 1*(data$ind2==3);
data$ind.manufacture <- 1*(data$ind2==4);
data$ind.retail <- 1*(data$ind2==5);
data$ind.transport <- 1*(data$ind2==6);
data$ind.information <- 1*(data$ind2==7);
data$ind.finance <- 1*(data$ind2==8);
data$ind.professional <- 1*(data$ind2==9);
data$ind.education <- 1*(data$ind2==10);
data$ind.leisure <- 1*(data$ind2==11);
data$ind.services <- 1*(data$ind2==12);
data$ind.public <- 1*(data$ind2==13);

# create education factor;
data$educ <- factor(data$lhs+2*data$hsg+3*data$sc+4*data$cg+5*data$ad);

attach(data);
n<-dim(data)[1];
us    <- c(2:98)/100; # percentiles of interest for SPE
taus  <- c(2:98)/100; # indexes for QR
alpha  <- .1; #Significance level
beta   <- .1; #Tail quantile index for classication analysis
R      <- 500; #number of bootstrap replications

########## Descriptive statistics

# Function to compute weighted std dev;

weighted.sd <- function(x, weights) {return(sqrt(wtd.var(x, weights, normwt = TRUE, na.rm = TRUE)))};

vars <- c("lnw","female","married","widowed","separated","divorced","nevermarried","lhs","hsg","sc","cg","ad","ne","mw","so","we",
          "exp1","occ.manager","occ.service","occ.sales","occ.construction","occ.production", "ind.minery", "ind.construction", 
          "ind.manufacture", "ind.retail", "ind.transport", "ind.information", "ind.finance", "ind.professional", 
          "ind.education", "ind.leisure", "ind.services", "ind.public");
options(digits=2);
dstats <- cbind(sapply(data[,vars], weighted.mean, weight), sapply(data[,vars], weighted.sd, weight), apply(data[female==1,vars], 2, weighted.mean, weight[female==1]), apply(data[female==0,vars], 2, weighted.mean, weight[female==0]));

xtable(dstats);

###############################################################################################
# FULL SPECIFICATION
###############################################################################################
 
# Part 1: Unconditional effects;

fmla1      <- lnw ~ female* (widowed + divorced + separated + nevermarried + (exp1+exp2+exp3+exp4)*(educ+occ2+ind2) + educ*(occ2+ind2) + mw + so + we);
# Introduce formula manually to avoid singularities in rq, drop interactions female:educ5:ind27 (female:ad:ind.information), female:educ5:ind213 (female:ad:ind.public)
fmla1      <- lnw ~ female + widowed + divorced + separated + nevermarried + (exp1+exp2+exp3+exp4)*(educ+occ2+ind2) + educ*(occ2+ind2) + mw + so + we +  
            female:(widowed + divorced + separated + nevermarried + (exp1+exp2+exp3+exp4)*(educ+occ2+ind2) + educ + occ2 + ind2 + educ:occ2 +
            factor(hsg*ind.minery) + factor(hsg*ind.construction) + factor(hsg*ind.manufacture) +  factor(hsg*ind.retail) + factor(hsg*ind.transport) +  factor(hsg*ind.information) +
            factor(hsg*ind.finance) + factor(hsg*ind.professional) + factor(hsg*ind.education) + factor(hsg*ind.leisure) +  factor(hsg*ind.services) +   
            factor(sc*ind.minery) + factor(sc*ind.construction) + factor(sc*ind.manufacture) +  factor(sc*ind.retail) + factor(sc*ind.transport) +  factor(sc*ind.information) +
            factor(sc*ind.finance) + factor(sc*ind.professional) + factor(sc*ind.education) + factor(sc*ind.leisure) +  factor(sc*ind.services) +   
            factor(cg*ind.minery) + factor(cg*ind.construction) + factor(cg*ind.manufacture) +  factor(cg*ind.retail) + factor(cg*ind.transport) +  factor(cg*ind.information) +
            factor(cg*ind.finance) + factor(cg*ind.professional) + factor(cg*ind.education) + factor(cg*ind.leisure) +  factor(cg*ind.services) +   
            factor(ad*ind.minery) + factor(ad*ind.construction) + factor(ad*ind.manufacture) +  factor(ad*ind.retail) + factor(ad*ind.transport) +  
            factor(ad*ind.finance) + factor(ad*ind.professional) + factor(ad*ind.education) + factor(ad*ind.leisure)  +
            mw + so + we);

qr.fit     <- rq.nostop(fmla1, weights = weight, tau = taus, method = "sfn2",  control = list(tmpmax = 1e6))

## female = 0 design matrix
d0<- qr.fit$model;
d0[,2]<- rep(0,n);
## female = 1 design matrix
d1<- qr.fit$model;
d1[,2]<- rep(1,n);

PE      <- predict(qr.fit, newdata=d1) -  predict(qr.fit, newdata=d0); #PEs

# Inference by bootstrap;

# Fuction to estimate weighted distribution;

wcdf <- function(y, x, w)
{;
  Ff    <- weighted.mean((x <= y), w); 
  return(Ff);
};

# Function to generate bootstrap data sets;

data.rg <- function(data, mle)
{
  bdata <- data;
  n<-dim(data)[1];
  multipliers  <- rexp(n);
  bdata$weight <- data$weight * multipliers / sum(multipliers);
  return(bdata);
}


########## statistics to be computed in each bootstrap draw #####################################
boot.SE<- function(data, fmla1, us, vars, lexper, lwage){
  n<-dim(data)[1]
  qr.fit <- rq.nostop(fmla1, data=data, weights = weight, tau = taus, method = "sfn2",  control = list(tmpmax = 1e6));
  ## design matrices for PE
  d0<- qr.fit$model;  
  d0[,2]<- rep(0,n);
  d1<- qr.fit$model;  
  d1[,2]<- rep(1,n);
  # PEs
  PE.qr    <- predict(qr.fit, newdata=d1) -  predict(qr.fit, newdata=d0);
  rm(d1,d0);
  PE       <- matrix(PE.qr, ncol = 1, byrow = FALSE);  
  PET      <- PE.qr[data$female==1, ];
  PET      <- matrix(PET, ncol = 1, byrow = FALSE);
  PET.w    <- matrix(data$weight[data$female==1], ncol = 1, nrow = nrow(PET), byrow = FALSE);
  
  # Means of high and low affected;
  high.effect<- wtd.quantile(PET, PET.w, 1-beta);
  low.effect<- wtd.quantile(PET, PET.w, beta);
  bigdata  <- kronecker(matrix(1,nrow = length(taus)), cbind(as.matrix(data[female==1,vars])));
  low.affected<- bigdata[PET>=high.effect, ];
  high.affected<- bigdata[PET<=low.effect, ];
  rm(bigdata);
  mean.ha <- apply(low.affected, 2, weighted.mean, PET.w[PET>=high.effect], na.rm=TRUE);
  mean.la <- apply(high.affected, 2, weighted.mean, PET.w[PET<=low.effect], na.rm=TRUE);
  
  # SDs of high and low affected;
  sd.ha <- apply(low.affected, 2, weighted.sd, PET.w[PET>=high.effect]);
  sd.la <- apply(high.affected, 2, weighted.sd, PET.w[PET<=low.effect]);
  
  # Experience CDFs of high and low affected;
  colnames(low.affected)<- vars;
  colnames(high.affected)<- vars;
  cdf.exper.ha <- sapply(lexper, wcdf, x = low.affected[,'exp1'],   w = PET.w[PET>=high.effect]);
  cdf.exper.la <- sapply(lexper, wcdf, x = high.affected[,'exp1'],  w = PET.w[PET<=low.effect]);  
  
  # Log wage CDFs of high and low affected;
  cdf.lnw.ha <- sapply(lwage, wcdf, x = low.affected[,'lnw'],   w = PET.w[PET>=high.effect]);
  cdf.lnw.la <- sapply(lwage, wcdf, x = high.affected[,'lnw'],  w = PET.w[PET<=low.effect]);  
  
  # PEs by education;
  PE.hs <- matrix(PE[data$lhs + data$hsg==1 & data$female==1, ], ncol = 1, byrow = FALSE);
  w.hs  <- matrix(data$weight[data$lhs + data$hsg==1 & data$female==1], ncol = 1, nrow = nrow(PE.hs), byrow = FALSE);  
  PE.sc <- matrix(PE[data$sc==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.sc  <- matrix(data$weight[data$sc==1 & data$female==1], ncol = 1, nrow = nrow(PE.sc), byrow = FALSE);  
  PE.cg <- matrix(PE[data$cg==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.cg  <- matrix(data$weight[data$cg==1 & data$female==1], ncol = 1, nrow = nrow(PE.cg), byrow = FALSE);  
  PE.ad <- matrix(PE[data$ad==1 & data$female==1, ], ncol = 1, byrow = FALSE);
  w.ad  <- matrix(data$weight[data$ad==1 & data$female==1], ncol = 1, nrow = nrow(PE.ad), byrow = FALSE);
  m.hs <- weighted.mean(PE.hs, w.hs);
  q.hs <- wtd.quantile(PE.hs, w.hs, us);
  m.sc <- weighted.mean(PE.sc, w.sc);
  q.sc <- wtd.quantile(PE.sc, w.sc, us);
  m.cg <- weighted.mean(PE.cg, w.cg);
  q.cg <- wtd.quantile(PE.cg, w.cg, us);
  m.ad <- weighted.mean(PE.ad, w.ad);
  q.ad <- wtd.quantile(PE.ad, w.ad, us);
  rm(PE.hs,w.hs,PE.sc,w.sc,PE.cg,w.cg,PE.ad,w.ad);
  
  
  # PEs by marital status;
  PE.nm <- matrix(PE[data$nevermarried==1 & data$female==1, ], ncol = 1, byrow = FALSE);
  w.nm  <- matrix(data$weight[data$nevermarried==1 & data$female==1], ncol = 1, nrow = nrow(PE.nm), byrow = FALSE);  
  PE.ma <- matrix(PE[data$married==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.ma  <- matrix(data$weight[data$married==1 & data$female==1], ncol = 1, nrow = nrow(PE.ma), byrow = FALSE);  
  PE.di <- matrix(PE[data$divorced==1 & data$female==1, ], ncol = 1, byrow = FALSE);
  w.di  <- matrix(data$weight[data$divorced==1 & data$female==1], ncol = 1, nrow = nrow(PE.di), byrow = FALSE); 
  m.nm <- weighted.mean(PE.nm, w.nm);
  q.nm <- wtd.quantile(PE.nm, w.nm, us);
  m.ma <- weighted.mean(PE.ma, w.ma);
  q.ma <- wtd.quantile(PE.ma, w.ma, us);
  m.di <- weighted.mean(PE.di, w.di);
  q.di <- wtd.quantile(PE.di, w.di, us);
  rm(PE.nm,w.nm,PE.ma,w.ma,PE.di,w.di);
  
  # PEs by experience;
  le <- (data$exp1 <= wtd.quantile(data$exp1[data$female==1], data$weight, 1/4));
  me <- (data$exp1 > wtd.quantile(data$exp1[data$female==1], data$weight, 1/4) & data$exp1 < wtd.quantile(data$exp1[data$female==1], data$weight, 3/4));
  he <- (data$exp1 >= wtd.quantile(data$exp1[data$female==1], data$weight, 3/4));
  PE.le <- matrix(PE[le==1 & data$female==1, ], ncol = 1, byrow = FALSE);
  w.le  <- matrix(data$weight[le==1 & data$female==1], ncol = 1, nrow = nrow(PE.le), byrow = FALSE);  
  PE.me <- matrix(PE[me==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.me  <- matrix(data$weight[me==1 & data$female==1], ncol = 1, nrow = nrow(PE.me), byrow = FALSE);  
  PE.he <- matrix(PE[he==1 & data$female==1, ], ncol = 1, byrow = FALSE);
  w.he  <- matrix(data$weight[he==1 & data$female==1], ncol = 1, nrow = nrow(PE.he), byrow = FALSE);  
  m.le <- weighted.mean(PE.le, w.le);
  q.le <- wtd.quantile(PE.le, w.le, us);
  m.me <- weighted.mean(PE.me, w.me);
  q.me <- wtd.quantile(PE.me, w.me, us);
  m.he <- weighted.mean(PE.he, w.he);
  q.he <- wtd.quantile(PE.he, w.he, us);
  rm(le,me,he,PE.le,w.le,PE.me,w.me,PE.he,w.he);
  
  # PEs by occupation;
  PE.o1 <- matrix(PE[data$occ.manager==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.o1  <- matrix(data$weight[data$occ.manager==1 & data$female==1], ncol = 1, nrow = nrow(PE.o1), byrow = FALSE);  
  PE.o2 <- matrix(PE[data$occ.sales==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.o2  <- matrix(data$weight[data$occ.sales==1 & data$female==1], ncol = 1, nrow = nrow(PE.o2), byrow = FALSE);  
  PE.o3 <- matrix(PE[data$occ.service==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.o3  <- matrix(data$weight[data$occ.service==1 & data$female==1], ncol = 1, nrow = nrow(PE.o3), byrow = FALSE);  
  PE.o4 <- matrix(PE[data$occ.production==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.o4  <- matrix(data$weight[data$occ.production==1 & data$female==1], ncol = 1, nrow = nrow(PE.o4), byrow = FALSE);  
  m.o1 <- weighted.mean(PE.o1, w.o1);
  q.o1 <- wtd.quantile(PE.o1, w.o1, us);
  m.o2 <- weighted.mean(PE.o2, w.o2);
  q.o2 <- wtd.quantile(PE.o2, w.o2, us);
  m.o3 <- weighted.mean(PE.o3, w.o3);
  q.o3 <- wtd.quantile(PE.o3, w.o3, us);
  m.o4 <- weighted.mean(PE.o4, w.o4);
  q.o4 <- wtd.quantile(PE.o4, w.o4, us);
  rm(PE.o1,w.o1,PE.o2,w.o2,PE.o3,w.o3,PE.o4,w.o4);
  
  # PEs by occupation;
  PE.i1 <- matrix(PE[data$ind.manufacture==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.i1  <- matrix(data$weight[data$ind.manufacture==1 & data$female==1], ncol = 1, nrow = nrow(PE.i1), byrow = FALSE);  
  PE.i2 <- matrix(PE[data$ind.finance==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.i2  <- matrix(data$weight[data$ind.finance==1 & data$female==1], ncol = 1, nrow = nrow(PE.i2), byrow = FALSE);  
  PE.i3 <- matrix(PE[data$ind.professional==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.i3  <- matrix(data$weight[data$ind.professional==1 & data$female==1], ncol = 1, nrow = nrow(PE.i3), byrow = FALSE);  
  PE.i4 <- matrix(PE[data$ind.education==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.i4  <- matrix(data$weight[data$ind.education==1 & data$female==1], ncol = 1, nrow = nrow(PE.i4), byrow = FALSE);  
  PE.i5 <- matrix(PE[data$ind.leisure==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.i5  <- matrix(data$weight[data$ind.leisure==1 & data$female==1], ncol = 1, nrow = nrow(PE.i5), byrow = FALSE);  
  PE.i6 <- matrix(PE[data$ind.public==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.i6  <- matrix(data$weight[data$ind.public==1 & data$female==1], ncol = 1, nrow = nrow(PE.i6), byrow = FALSE);  
  PE.i7 <- matrix(PE[data$ind.services==1 & data$female==1, ], ncol = 1, byrow = FALSE);  
  w.i7  <- matrix(data$weight[data$ind.services==1 & data$female==1], ncol = 1, nrow = nrow(PE.i7), byrow = FALSE);  
  m.i1 <- weighted.mean(PE.i1, w.i1);
  q.i1 <- wtd.quantile(PE.i1, w.i1, us);
  m.i2 <- weighted.mean(PE.i2, w.i2);
  q.i2 <- wtd.quantile(PE.i2, w.i2, us);
  m.i3 <- weighted.mean(PE.i3, w.i3);
  q.i3 <- wtd.quantile(PE.i3, w.i3, us);
  m.i4 <- weighted.mean(PE.i4, w.i4);
  q.i4 <- wtd.quantile(PE.i4, w.i4, us);
  m.i5 <- weighted.mean(PE.i5, w.i5);
  q.i5 <- wtd.quantile(PE.i5, w.i5, us);
  m.i6 <- weighted.mean(PE.i6, w.i6);
  q.i6 <- wtd.quantile(PE.i6, w.i6, us);
  m.i7 <- weighted.mean(PE.i7, w.i7);
  q.i7 <- wtd.quantile(PE.i7, w.i7, us);
  rm(PE.i1,w.i1,PE.i2,w.i2,PE.i3,w.i3,PE.i4,w.i4,PE.i5,w.i5,PE.i6,w.i6,PE.i7,w.i7);
  
  return(c(mean.ha, mean.la, sd.ha, sd.la, cdf.exper.ha, cdf.exper.la, cdf.lnw.ha, cdf.lnw.la,
           weighted.mean(PET, PET.w), wtd.quantile(PET, PET.w, us),
           m.hs, q.hs, m.sc, q.sc, m.cg, q.cg, m.ad, q.ad,
           m.nm, q.nm, m.ma, q.ma, m.di, q.di,
           m.le, q.le, m.me, q.me, m.he, q.he,
           m.o1, q.o1, m.o2, q.o2, m.o3, q.o3, m.o4, q.o4,
           m.i1, q.i1, m.i2, q.i2, m.i3, q.i3, m.i4, q.i4, m.i5, q.i5, m.i6, q.i6, m.i7, q.i7));
}

set.seed(1);
result.boot.SE.qr <- boot(data = data, statistic=boot.SE, sim = "parametric", ran.gen = data.rg, mle = 0, fmla1 = fmla1, us = us, vars = vars, lexper = sort(unique(exp1)), 
                          lwage = sort(unique(wtd.quantile(lnw, weight, c(2:98)/100))), parallel="multicore", ncpus = 16, R=R);

###########################################################################################################;
# Part 1: Standard Errors and Confidence bands for Means and SDs in Classification Analysis based on SPET;
###########################################################################################################;

PET      <- matrix(PE[which(female==1),], ncol = 1, byrow = FALSE);  
PET.w    <- matrix(weight[female==1], ncol = 1, nrow = nrow(PET), byrow = FALSE);

high.effect<- wtd.quantile(PET, PET.w, 1-beta);
low.effect<- wtd.quantile(PET, PET.w, beta);
print(high.effect); print(low.effect);

bigdata  <- kronecker(matrix(1,nrow = length(taus)), cbind(as.matrix(data[female==1,vars])));
low.affected<- bigdata[PET>=high.effect, ];
high.affected<- bigdata[PET<=low.effect, ];
rm(bigdata);

# Biases and Standard errors;
draws <- result.boot.SE.qr$t[,1:(2*length(vars))];  #these are bs draws for the means
bse   <- (apply(draws, 2, quantile, .75, na.rm=TRUE) - apply(draws, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
bm    <- apply(draws, 2, mean, na.rm=TRUE);
draws.sd <- result.boot.SE.qr$t[,2*length(vars) + 1:(2*length(vars))];  #these are bs draws for the sds
bse.sd   <- (apply(draws.sd, 2, quantile, .75, na.rm=TRUE) - apply(draws.sd, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
bm.sd    <- apply(draws.sd, 2, mean, na.rm=TRUE);

# Table comparing corrected and uncorrected (for threshold estimation) std errors for the means
table.who<- matrix(0,6,length(vars));
colnames(table.who)<- vars;
table.who[1,]<- apply(low.affected, 2, weighted.mean, PET.w[PET>=high.effect]);
table.who[2,]<- bse[c(1:length(vars))];
table.who[3,]<- apply(low.affected, 2, weighted.sd, PET.w[PET>=high.effect])/sqrt(nrow(low.affected));
table.who[4,]<- apply(high.affected, 2, weighted.mean, PET.w[PET<=low.effect]);
table.who[5,]<- bse[-c(1:length(vars))];
table.who[6,]<- apply(high.affected, 2, weighted.sd, PET.w[PET<=low.effect])/sqrt(nrow(high.affected));

xtable(t(table.who), digits=2);

xtable(t(table.who[-c(3,6),]), digits=2);

# Bias Corrected Table comparing corrected and uncorrected (for threshold estimation) std errors for the means
table.who.bc<- matrix(0,6,length(vars));
colnames(table.who.bc)<- vars;
table.who.bc[1,]<- 2*apply(low.affected, 2, weighted.mean, PET.w[PET>=high.effect]) - bm[c(1:length(vars))];
table.who.bc[2,]<- bse[c(1:length(vars))];
table.who.bc[3,]<- 2*apply(low.affected, 2, weighted.sd, PET.w[PET>=high.effect]) - bm.sd[c(1:length(vars))];
table.who.bc[4,]<- 2*apply(high.affected, 2, weighted.mean, PET.w[PET<=low.effect]) - bm[-c(1:length(vars))];
table.who.bc[5,]<- bse[-c(1:length(vars))];
table.who.bc[6,]<- 2*apply(high.affected, 2, weighted.sd, PET.w[PET<=low.effect]) - bm.sd[-c(1:length(vars))];

xtable(t(table.who.bc), digits=2);

xtable(t(table.who.bc[-c(3,6),]), digits=2);



# Table for the mean and std dev including estimates and std errors

table.full.who<- matrix(0,8,length(vars));
colnames(table.full.who)<- vars;
table.full.who[1,]<- apply(low.affected, 2, weighted.mean, PET.w[PET>=high.effect]);
table.full.who[2,]<- bse[c(1:length(vars))];
table.full.who[3,]<- apply(low.affected, 2, weighted.sd, PET.w[PET>=high.effect]);
table.full.who[4,]<- bse.sd[c(1:length(vars))];
table.full.who[5,]<- apply(high.affected, 2, weighted.mean, PET.w[PET<=low.effect]);
table.full.who[6,]<- bse[-c(1:length(vars))];
table.full.who[7,]<- apply(high.affected, 2, weighted.sd, PET.w[PET<=low.effect]);
table.full.who[8,]<- bse.sd[-c(1:length(vars))];

xtable(t(table.full.who), digits=2);

# Bias Corrected Table for the mean and std dev including estimates and std errors

table.full.who.bc<- matrix(0,8,length(vars));
colnames(table.full.who.bc)<- vars;
table.full.who.bc[1,]<- 2*apply(low.affected, 2, weighted.mean, PET.w[PET>=high.effect]) - bm[c(1:length(vars))];
table.full.who.bc[2,]<- bse[c(1:length(vars))];
table.full.who.bc[3,]<- 2*apply(low.affected, 2, weighted.sd, PET.w[PET>=high.effect]) - bm.sd[c(1:length(vars))];
table.full.who.bc[4,]<- bse.sd[c(1:length(vars))];
table.full.who.bc[5,]<- 2*apply(high.affected, 2, weighted.mean, PET.w[PET<=low.effect]) - bm[-c(1:length(vars))];
table.full.who.bc[6,]<- bse[-c(1:length(vars))];
table.full.who.bc[7,]<- 2*apply(high.affected, 2, weighted.sd, PET.w[PET<=low.effect]) - bm.sd[-c(1:length(vars))];
table.full.who.bc[8,]<- bse.sd[-c(1:length(vars))];

xtable(t(table.full.who.bc), digits=2);


rm(draws.sd, bse.sd);

##################################################################################################;
# Classification Analysis: differences between  more and less affected;
##################################################################################################;

draws.ha <- draws[,1:length(vars)];
draws.la <- draws[,-c(1:length(vars))];
colnames(draws.ha) <- vars;
colnames(draws.la) <- vars;

rm(d0, d1, draws);

library(ggplot2); # library for fancy plots;

##########;
# Region;
##########;

vars.sub <- c("ne","mw","so","we");
means.affected <- data.frame(affected <- c(rep(0,length(vars.sub)), rep(1,length(vars.sub))), variable = rep(vars.sub,2), means = c(table.who[1,vars.sub], table.who[4,vars.sub]),
                             ses = c(table.who[2,vars.sub], table.who[5,vars.sub]));

delta.d <- (draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)]) - matrix(table.who[1,vars.sub] - table.who[4,vars.sub], nrow = R, ncol = length(vars.sub), byrow=TRUE);
bse.d   <- (apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .75, na.rm=TRUE) - apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
zs      <- apply(abs(delta.d /matrix(bse.d, nrow = R, ncol = length(vars.sub), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
zs.d    <- abs(table.who[1,vars.sub] - table.who[4,vars.sub])/bse.d;
# Function to obtain empirical p-values;
epvals <- function(z,zs)
{;
  return(mean(zs>z));
};
pvals <- sapply(zs.d, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 
zs.d.bc  <- abs(table.who.bc[1,vars.sub] - table.who.bc[4,vars.sub])/bse.d;
pvals.bc <- sapply(zs.d.bc, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 

table.who.d <- matrix(0, ncol = 4, nrow = length(vars));
rownames(table.who.d) <- vars;
colnames(table.who.d) <- c("Est.", "Std.Err.", "P-value","Joint P-val.");
table.who.d[vars.sub, 1] <- table.who[4,vars.sub] - table.who[1,vars.sub];
table.who.d[vars.sub, 2] <- bse.d;
table.who.d[vars.sub, 3] <- pvals;

table.who.d.bc <- matrix(0, ncol = 4, nrow = length(vars));
rownames(table.who.d.bc) <- vars;
colnames(table.who.d.bc) <- c("Est.", "Std.Err.", "P-value","Joint P-val.");
table.who.d.bc[vars.sub, 1] <- table.who.bc[4,vars.sub] - table.who.bc[1,vars.sub];
table.who.d.bc[vars.sub, 2] <- bse.d;
table.who.d.bc[vars.sub, 3] <- pvals.bc;

rm(delta.d,bse.d,zs,z.d,pval,z.d.bc,pval.bc);


#####################;
# By marital status;
#####################;

vars.sub <- c("married","widowed","separated","divorced","nevermarried")
means.affected <- data.frame(affected <- c(rep(0,length(vars.sub)), rep(1,length(vars.sub))), variable = rep(vars.sub,2), means = c(table.who[1,vars.sub], table.who[4,vars.sub]),
                             ses = c(table.who[2,vars.sub], table.who[5,vars.sub]));

delta.d <- (draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)]) - matrix(table.who[1,vars.sub] - table.who[4,vars.sub], nrow = R, ncol = length(vars.sub), byrow=TRUE);
bse.d   <- (apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .75, na.rm=TRUE) - apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
zs      <- apply(abs(delta.d /matrix(bse.d, nrow = R, ncol = length(vars.sub), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
zs.d    <- abs(table.who[1,vars.sub] - table.who[4,vars.sub])/bse.d;
pvals <- sapply(zs.d, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 
zs.d.bc  <- abs(table.who.bc[1,vars.sub] - table.who.bc[4,vars.sub])/bse.d;
pvals.bc <- sapply(zs.d.bc, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 

table.who.d[vars.sub, 1] <- table.who[4,vars.sub] - table.who[1,vars.sub];
table.who.d[vars.sub, 2] <- bse.d;
table.who.d[vars.sub, 3] <- pvals;

table.who.d.bc[vars.sub, 1] <- table.who.bc[4,vars.sub] - table.who.bc[1,vars.sub];
table.who.d.bc[vars.sub, 2] <- bse.d;
table.who.d.bc[vars.sub, 3] <- pvals.bc;

################;
# By education;
################;

vars.sub <- c("lhs","hsg","sc","cg","ad")
means.affected <- data.frame(affected <- c(rep(0,length(vars.sub)), rep(1,length(vars.sub))), variable = rep(vars.sub,2), means = c(table.who[1,vars.sub], table.who[4,vars.sub]),
                             ses = c(table.who[2,vars.sub], table.who[5,vars.sub]));

delta.d <- (draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)]) - matrix(table.who[1,vars.sub] - table.who[4,vars.sub], nrow = R, ncol = length(vars.sub), byrow=TRUE);
bse.d   <- (apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .75, na.rm=TRUE) - apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
zs      <- apply(abs(delta.d /matrix(bse.d, nrow = R, ncol = length(vars.sub), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
zs.d    <- abs(table.who[1,vars.sub] - table.who[4,vars.sub])/bse.d;
pvals <- sapply(zs.d, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 
zs.d.bc  <- abs(table.who.bc[1,vars.sub] - table.who.bc[4,vars.sub])/bse.d;
pvals.bc <- sapply(zs.d.bc, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 

table.who.d[vars.sub, 1] <- table.who[4,vars.sub] - table.who[1,vars.sub];
table.who.d[vars.sub, 2] <- bse.d;
table.who.d[vars.sub, 3] <- pvals;

table.who.d.bc[vars.sub, 1] <- table.who.bc[4,vars.sub] - table.who.bc[1,vars.sub];
table.who.d.bc[vars.sub, 2] <- bse.d;
table.who.d.bc[vars.sub, 3] <- pvals.bc;

rm(delta.d,bse.d,zs,z.d,pval,z.d.bc,pval.bc);

#################;
# By occupation;
#################;

vars.sub <- c("occ.manager","occ.service","occ.sales","occ.construction","occ.production")
means.affected <- data.frame(affected <- c(rep(0,length(vars.sub)), rep(1,length(vars.sub))), variable = rep(substr(vars.sub,5,20),2), means = c(table.who[1,vars.sub], table.who[4,vars.sub]),
                             ses = c(table.who[2,vars.sub], table.who[5,vars.sub]));

delta.d <- (draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)]) - matrix(table.who[1,vars.sub] - table.who[4,vars.sub], nrow = R, ncol = length(vars.sub), byrow=TRUE);
bse.d   <- (apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .75, na.rm=TRUE) - apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
zs      <- apply(abs(delta.d /matrix(bse.d, nrow = R, ncol = length(vars.sub), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
zs.d    <- abs(table.who[1,vars.sub] - table.who[4,vars.sub])/bse.d;
pvals <- sapply(zs.d, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 
zs.d.bc  <- abs(table.who.bc[1,vars.sub] - table.who.bc[4,vars.sub])/bse.d;
pvals.bc <- sapply(zs.d.bc, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 

table.who.d[vars.sub, 1] <- table.who[4,vars.sub] - table.who[1,vars.sub];
table.who.d[vars.sub, 2] <- bse.d;
table.who.d[vars.sub, 3] <- pvals;

table.who.d.bc[vars.sub, 1] <- table.who.bc[4,vars.sub] - table.who.bc[1,vars.sub];
table.who.d.bc[vars.sub, 2] <- bse.d;
table.who.d.bc[vars.sub, 3] <- pvals.bc;

rm(delta.d,bse.d,zs,z.d,pval,z.d.bc,pval.bc);


###############;
# By industry;
###############;

vars.sub <- c("ind.minery", "ind.construction", 
              "ind.manufacture", "ind.retail", "ind.transport", "ind.information", "ind.finance", "ind.professional", 
              "ind.education", "ind.leisure", "ind.services", "ind.public")
means.affected <- data.frame(affected <- c(rep(0,length(vars.sub)), rep(1,length(vars.sub))), variable = rep(substr(vars.sub,5,10),2), means = c(table.who[1,vars.sub], table.who[4,vars.sub]),
                             ses = c(table.who[2,vars.sub], table.who[5,vars.sub]));

delta.d <- (draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)]) - matrix(table.who[1,vars.sub] - table.who[4,vars.sub], nrow = R, ncol = length(vars.sub), byrow=TRUE);
bse.d   <- (apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .75, na.rm=TRUE) - apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
zs      <- apply(abs(delta.d /matrix(bse.d, nrow = R, ncol = length(vars.sub), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
zs.d    <- abs(table.who[1,vars.sub] - table.who[4,vars.sub])/bse.d;
pvals <- sapply(zs.d, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 
zs.d.bc  <- abs(table.who.bc[1,vars.sub] - table.who.bc[4,vars.sub])/bse.d;
pvals.bc <- sapply(zs.d.bc, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 

table.who.d[vars.sub, 1] <- table.who[4,vars.sub] - table.who[1,vars.sub];
table.who.d[vars.sub, 2] <- bse.d;
table.who.d[vars.sub, 3] <- pvals;

table.who.d.bc[vars.sub, 1] <- table.who.bc[4,vars.sub] - table.who.bc[1,vars.sub];
table.who.d.bc[vars.sub, 2] <- bse.d;
table.who.d.bc[vars.sub, 3] <- pvals.bc;

rm(delta.d,bse.d,zs,z.d,pval,z.d.bc,pval.bc);


#################;
# By experience;
#################;

vars.sub <- c("exp1")

delta.d <- (draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)]) - matrix(table.who[1,vars.sub] - table.who[4,vars.sub], nrow = R, ncol = length(vars.sub), byrow=TRUE);
bse.d   <- (quantile(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], .75, na.rm=TRUE) - quantile(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
zs      <- abs(delta.d /matrix(bse.d, nrow = R, ncol = length(vars.sub), byrow=TRUE));  #abs t-stat
z.d     <- max(abs(table.who[1,vars.sub] - table.who[4,vars.sub])/bse.d);
pvals    <- mean(zs>z.d);  #critical value
z.d.bc  <- max(abs(table.who.bc[1,vars.sub] - table.who.bc[4,vars.sub])/bse.d);
pvals.bc <- mean(zs>z.d.bc);  #critical value

table.who.d[vars.sub, 1] <- table.who[4,vars.sub] - table.who[1,vars.sub];
table.who.d[vars.sub, 2] <- bse.d;
table.who.d[vars.sub, 3] <- pvals;

table.who.d.bc[vars.sub, 1] <- table.who.bc[4,vars.sub] - table.who.bc[1,vars.sub];
table.who.d.bc[vars.sub, 2] <- bse.d;
table.who.d.bc[vars.sub, 3] <- pvals.bc;

rm(delta.d,bse.d,zs,z.d,pval,z.d.bc,pval.bc);


# CDF of experience;
 
lexper <- sort(unique(exp1));
colnames(low.affected)<- c(vars);
colnames(high.affected)<- c(vars);
cdf.exper.ha <- sapply(lexper, wcdf, x = low.affected[,'exp1'],   w = PET.w[PET>=high.effect]);
cdf.exper.la <- sapply(lexper, wcdf, x = high.affected[,'exp1'],  w = PET.w[PET<=low.effect]);  

draws <- result.boot.SE.qr$t[,4*length(vars) + 1:(2*length(lexper))];  #these are bs draws for the cdfs
bse   <- (apply(draws, 2, quantile, .75, na.rm=TRUE) - apply(draws, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust se estimator
bse[bse==0] <- NA;
delta    <- draws-matrix(c(cdf.exper.ha, cdf.exper.la), nrow = nrow(draws), ncol = ncol(draws), byrow=TRUE);
trim.tails <- (lexper >= quantile(exp1,.02) & lexper <= quantile(exp1,.98)); # trims the tails
zs<- apply(abs(delta /matrix(bse, nrow = nrow(draws), ncol = ncol(draws), byrow=TRUE))[,trim.tails], 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

bse   <- (apply(draws, 2, quantile, .75, na.rm=TRUE) - apply(draws, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust se estimator
ucdf.exper.ha <- cdf.exper.ha + crt*bse[1:length(lexper)];
lcdf.exper.ha <- cdf.exper.ha - crt*bse[1:length(lexper)];
ucdf.exper.la <- cdf.exper.la + crt*bse[-(1:length(lexper))];
lcdf.exper.la <- cdf.exper.la - crt*bse[-(1:length(lexper))];

ucdf.exper.ha <- sort(ucdf.exper.ha * (ucdf.exper.ha > 0 & ucdf.exper.ha < 1) + (ucdf.exper.ha >= 1)); # Imposing shape restrictions
lcdf.exper.ha <- sort(lcdf.exper.ha * (lcdf.exper.ha > 0 & lcdf.exper.ha < 1) + (lcdf.exper.ha >= 1)); # Imposing shape restrictions
ucdf.exper.la <- sort(ucdf.exper.la * (ucdf.exper.la > 0 & ucdf.exper.la < 1) + (ucdf.exper.la >= 1)); # Imposing shape restrictions
lcdf.exper.la <- sort(lcdf.exper.la * (lcdf.exper.la > 0 & lcdf.exper.la < 1) + (lcdf.exper.la >= 1)); # Imposing shape restrictions

pdf("classification-EXP-qr.pdf", pointsize=15,width=8.0,height=8.0)
plot(lexper, cdf.exper.ha, type="n",col=4, xlab="Experience", lwd=2, 
     ylab="Probability", 
     #     main="Population",
     sub="Quantile model", 
     ylim=c(0,1))

polygon(c(lexper,rev(lexper)),c(ucdf.exper.la,rev(lcdf.exper.la)), density=60, border=F, 
        col='darkcyan', lty = 1, lwd = 1, type = "l");
lines(lexper, cdf.exper.la, lwd=2, col = 4, type = "l" )
polygon(c(lexper,rev(lexper)),c(ucdf.exper.ha,rev(lcdf.exper.ha)), density=60, border=F, 
        col='tomato', lty = 1, lwd = 1, type = "l");
lines(lexper, cdf.exper.ha, lwd=2, col = 2, type = "l")
legend( x="topleft", 
        legend=c("Most Affected","Least Affected","90% CB", "90% CB"), 
        col=c(4,2,"darkcyan","tomato"), lwd=c(1, 1, 5, 5), lty=c(1, 1, 1, 1), bty = 'n' )

dev.off();


# Bootstrap bias corrections;

bse   <- (apply(draws, 2, quantile, .75, na.rm=TRUE) - apply(draws, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust se estimator
bm    <- apply(draws, 2, mean, na.rm=TRUE); # Bias estimator
cdf.exper.ha.bc <- 2*cdf.exper.ha - bm[1:length(lexper)];
cdf.exper.la.bc <- 2*cdf.exper.la - bm[-(1:length(lexper))];
ucdf.exper.ha.bc <- 2*cdf.exper.ha - bm[1:length(lexper)] + crt*bse[1:length(lexper)];
lcdf.exper.ha.bc <- 2*cdf.exper.ha - bm[1:length(lexper)] - crt*bse[1:length(lexper)];
ucdf.exper.la.bc <- 2*cdf.exper.la - bm[-(1:length(lexper))] + crt*bse[-(1:length(lexper))];
lcdf.exper.la.bc <- 2*cdf.exper.la - bm[-(1:length(lexper))] - crt*bse[-(1:length(lexper))];

cdf.exper.ha.bc <- sort(cdf.exper.ha.bc * (cdf.exper.ha.bc > 0 & cdf.exper.ha.bc < 1) + (cdf.exper.ha.bc >= 1)); # Imposing shape restrictions
cdf.exper.la.bc <- sort(cdf.exper.la.bc * (cdf.exper.la.bc > 0 & cdf.exper.la.bc < 1) + (cdf.exper.la.bc >= 1)); # Imposing shape restrictions
ucdf.exper.ha.bc <- sort(ucdf.exper.ha.bc * (ucdf.exper.ha.bc > 0 & ucdf.exper.ha.bc < 1) + (ucdf.exper.ha.bc >= 1)); # Imposing shape restrictions
lcdf.exper.ha.bc <- sort(lcdf.exper.ha.bc * (lcdf.exper.ha.bc > 0 & lcdf.exper.ha.bc < 1) + (lcdf.exper.ha.bc >= 1)); # Imposing shape restrictions
ucdf.exper.la.bc <- sort(ucdf.exper.la.bc * (ucdf.exper.la.bc > 0 & ucdf.exper.la.bc < 1) + (ucdf.exper.la.bc >= 1)); # Imposing shape restrictions
lcdf.exper.la.bc <- sort(lcdf.exper.la.bc * (lcdf.exper.la.bc > 0 & lcdf.exper.la.bc < 1) + (lcdf.exper.la.bc >= 1)); # Imposing shape restrictions

pdf("classification-EXP-qr-bc.pdf", pointsize=15,width=8.0,height=8.0)
plot(lexper, cdf.exper.ha.bc, type="n",col=4, xlab="Experience", lwd=2, 
     ylab="Probability", 
     #     main="Population",
     sub="Quantile model", 
     ylim=c(0,1))

polygon(c(lexper,rev(lexper)),c(ucdf.exper.la.bc,rev(lcdf.exper.la.bc)), density=60, border=F, 
        col='darkcyan', lty = 1, lwd = 1, type = "l");
lines(lexper, cdf.exper.la.bc, lwd=2, col = 4, type = "l" )
polygon(c(lexper,rev(lexper)),c(ucdf.exper.ha.bc,rev(lcdf.exper.ha.bc)), density=60, border=F, 
        col='tomato', lty = 1, lwd = 1, type = "l");
lines(lexper, cdf.exper.ha.bc, lwd=2, col = 2, type = "l")
legend( x="topleft", 
        legend=c("Most Affected","Least Affected","90% CB", "90% CB"), 
        col=c(4,2,"darkcyan","tomato"), lwd=c(1, 1, 5, 5), lty=c(1, 1, 1, 1), bty = 'n' )

dev.off();


rm(cdf.exper.la, cdf.exper.ha, draws, bse, trim.tails, zs, delta, 
   ucdf.exper.la, ucdf.exper.ha, lcdf.exper.la, lcdf.exper.ha, ucdf.exper.la.bc, ucdf.exper.ha.bc, lcdf.exper.la.bc, lcdf.exper.ha.bc);

###########;
# By wage;
###########;

vars.sub <- c("lnw")

# Difference of means;
delta.d <- (draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)]) - matrix(table.who[1,vars.sub] - table.who[4,vars.sub], nrow = R, ncol = length(vars.sub), byrow=TRUE);
bse.d   <- (quantile(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], .75, na.rm=TRUE) - quantile(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
zs      <- abs(delta.d /matrix(bse.d, nrow = R, ncol = length(vars.sub), byrow=TRUE));  #abs t-stat
z.d     <- max(abs(table.who[1,vars.sub] - table.who[4,vars.sub])/bse.d);
pvals    <- mean(zs>z.d);  #critical value
z.d.bc  <- max(abs(table.who.bc[1,vars.sub] - table.who.bc[4,vars.sub])/bse.d);
pvals.bc <- mean(zs>z.d.bc);  #critical value

table.who.d[vars.sub, 1] <- table.who[4,vars.sub] - table.who[1,vars.sub];
table.who.d[vars.sub, 2] <- bse.d;
table.who.d[vars.sub, 3] <- pvals;

table.who.d.bc[vars.sub, 1] <- table.who.bc[4,vars.sub] - table.who.bc[1,vars.sub];
table.who.d.bc[vars.sub, 2] <- bse.d;
table.who.d.bc[vars.sub, 3] <- pvals.bc;

rm(delta.d,bse.d,zs,z.d,pval,z.d.bc,pval.bc);

# CDF of log wage;

lwage <- sort(unique(wtd.quantile(lnw, weight, c(2:98)/100)));
colnames(low.affected)<- c(vars);
colnames(high.affected)<- c(vars);
cdf.lnw.ha <- sapply(lwage, wcdf, x = low.affected[,'lnw'],   w = PET.w[PET>=high.effect]);
cdf.lnw.la <- sapply(lwage, wcdf, x = high.affected[,'lnw'],  w = PET.w[PET<=low.effect]);  

draws <- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 1:(2*length(lwage))];  #these are bs draws for the cdfs
bse   <- (apply(draws, 2, quantile, .75, na.rm=TRUE) - apply(draws, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust se estimator
bse[bse==0] <- NA;
delta    <- draws-matrix(c(cdf.lnw.ha, cdf.lnw.la), nrow = nrow(draws), ncol = ncol(draws), byrow=TRUE);
trim.tails <- (lwage >= quantile(lnw,.02) & lwage <= quantile(lnw,.98)); # trims the tails
zs<- apply(abs(delta /matrix(bse, nrow = nrow(draws), ncol = ncol(draws), byrow=TRUE))[,trim.tails], 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

bse   <- (apply(draws, 2, quantile, .75, na.rm=TRUE) - apply(draws, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust se estimator
ucdf.lnw.ha <- cdf.lnw.ha + crt*bse[1:length(lwage)];
lcdf.lnw.ha <- cdf.lnw.ha - crt*bse[1:length(lwage)];
ucdf.lnw.la <- cdf.lnw.la + crt*bse[-(1:length(lwage))];
lcdf.lnw.la <- cdf.lnw.la - crt*bse[-(1:length(lwage))];

ucdf.lnw.ha <- sort(ucdf.lnw.ha * (ucdf.lnw.ha > 0 & ucdf.lnw.ha < 1) + (ucdf.lnw.ha >= 1)); # Imposing shape restrictions
lcdf.lnw.ha <- sort(lcdf.lnw.ha * (lcdf.lnw.ha > 0 & lcdf.lnw.ha < 1) + (lcdf.lnw.ha >= 1)); # Imposing shape restrictions
ucdf.lnw.la <- sort(ucdf.lnw.la * (ucdf.lnw.la > 0 & ucdf.lnw.la < 1) + (ucdf.lnw.la >= 1)); # Imposing shape restrictions
lcdf.lnw.la <- sort(lcdf.lnw.la * (lcdf.lnw.la > 0 & lcdf.lnw.la < 1) + (lcdf.lnw.la >= 1)); # Imposing shape restrictions

pdf("classification-LW-qr.pdf", pointsize=15,width=8.0,height=8.0)
plot(lwage, cdf.lnw.ha, type="n",col=4, xlab="Log of Wage", lwd=2, 
     ylab="Probability", 
     #     main="Population",
     sub="Quantile model", 
     ylim=c(0,1))

polygon(c(lwage,rev(lwage)),c(ucdf.lnw.la,rev(lcdf.lnw.la)), density=60, border=F, 
        col='darkcyan', lty = 1, lwd = 1, type = "l");
lines(lwage, cdf.lnw.la, lwd=2, col = 4, type = "l" )
polygon(c(lwage,rev(lwage)),c(ucdf.lnw.ha,rev(lcdf.lnw.ha)), density=60, border=F, 
        col='tomato', lty = 1, lwd = 1, type = "l");
lines(lwage, cdf.lnw.ha, lwd=2, col = 2, type = "l")
legend( x="topleft", 
        legend=c("Most Affected","Least Affected","90% CB", "90% CB"), 
        col=c(4,2,"darkcyan","tomato"), lwd=c(1, 1, 5, 5), lty=c(1, 1, 1, 1), bty = 'n' )

dev.off();


# Bootstrap bias correction;

bse   <- (apply(draws, 2, quantile, .75, na.rm=TRUE) - apply(draws, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust se estimator
bm   <- apply(draws, 2, mean, na.rm=TRUE); # Bias estimator
cdf.lnw.ha.bc <- 2*cdf.lnw.ha - bm[1:length(lwage)];
cdf.lnw.la.bc <- 2*cdf.lnw.la - bm[-(1:length(lwage))];
ucdf.lnw.ha.bc <- 2*cdf.lnw.ha - bm[1:length(lwage)] + crt*bse[1:length(lwage)];
lcdf.lnw.ha.bc <- 2*cdf.lnw.ha - bm[1:length(lwage)] - crt*bse[1:length(lwage)];
ucdf.lnw.la.bc <- 2*cdf.lnw.la - bm[-(1:length(lwage))] + crt*bse[-(1:length(lwage))];
lcdf.lnw.la.bc <- 2*cdf.lnw.la - bm[-(1:length(lwage))] - crt*bse[-(1:length(lwage))];

cdf.lnw.ha.bc <- sort(cdf.lnw.ha.bc * (cdf.lnw.ha.bc > 0 & cdf.lnw.ha.bc < 1) + (cdf.lnw.ha.bc >= 1)); # Imposing shape restrictions
cdf.lnw.la.bc <- sort(cdf.lnw.la.bc * (cdf.lnw.la.bc > 0 & cdf.lnw.la.bc < 1) + (cdf.lnw.la.bc >= 1)); # Imposing shape restrictions
ucdf.lnw.ha.bc <- sort(ucdf.lnw.ha.bc * (ucdf.lnw.ha.bc > 0 & ucdf.lnw.ha.bc < 1) + (ucdf.lnw.ha.bc >= 1)); # Imposing shape restrictions
lcdf.lnw.ha.bc <- sort(lcdf.lnw.ha.bc * (lcdf.lnw.ha.bc > 0 & lcdf.lnw.ha.bc < 1) + (lcdf.lnw.ha.bc >= 1)); # Imposing shape restrictions
ucdf.lnw.la.bc <- sort(ucdf.lnw.la.bc * (ucdf.lnw.la.bc > 0 & ucdf.lnw.la.bc < 1) + (ucdf.lnw.la.bc >= 1)); # Imposing shape restrictions
lcdf.lnw.la.bc <- sort(lcdf.lnw.la.bc * (lcdf.lnw.la.bc > 0 & lcdf.lnw.la.bc < 1) + (lcdf.lnw.la.bc >= 1)); # Imposing shape restrictions

pdf("classification-LW-qr-bc.pdf", pointsize=15,width=8.0,height=8.0)
plot(lwage, cdf.lnw.ha.bc, type="n",col=4, xlab="Log of Wage", lwd=2, 
     ylab="Probability", 
     #     main="Population",
     sub="Quantile model", 
     ylim=c(0,1))

polygon(c(lwage,rev(lwage)),c(ucdf.lnw.la.bc,rev(lcdf.lnw.la.bc)), density=60, border=F, 
        col='darkcyan', lty = 1, lwd = 1, type = "l");
lines(lwage, cdf.lnw.la.bc, lwd=2, col = 4, type = "l" )
polygon(c(lwage,rev(lwage)),c(ucdf.lnw.ha.bc,rev(lcdf.lnw.ha.bc)), density=60, border=F, 
        col='tomato', lty = 1, lwd = 1, type = "l");
lines(lwage, cdf.lnw.ha.bc, lwd=2, col = 2, type = "l")
legend( x="topleft", 
        legend=c("Most Affected","Least Affected","90% CB", "90% CB"), 
        col=c(4,2,"darkcyan","tomato"), lwd=c(1, 1, 5, 5), lty=c(1, 1, 1, 1), bty = 'n' )

dev.off();

# Joint p-values for the differences of the means of all the variables

vars.sub <- c("lnw", "married","widowed","separated","divorced","nevermarried","lhs","hsg","sc","cg","ad","ne","mw","so","we",
              "exp1","occ.manager","occ.service","occ.sales","occ.construction","occ.production", "ind.minery", "ind.construction", 
              "ind.manufacture", "ind.retail", "ind.transport", "ind.information", "ind.finance", "ind.professional", 
              "ind.education", "ind.leisure", "ind.services", "ind.public");

delta.d <- (draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)]) - matrix(table.who[1,vars.sub] - table.who[4,vars.sub], nrow = R, ncol = length(vars.sub), byrow=TRUE);
bse.d   <- (apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .75, na.rm=TRUE) - apply(draws.ha[,rep(vars.sub)] - draws.la[,rep(vars.sub)], 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)); # Robust estimator
zs      <- apply(abs(delta.d /matrix(bse.d, nrow = R, ncol = length(vars.sub), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
zs.d    <- abs(table.who[1,vars.sub] - table.who[4,vars.sub])/bse.d;
pvals <- sapply(zs.d, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 
zs.d.bc  <- abs(table.who.bc[1,vars.sub] - table.who.bc[4,vars.sub])/bse.d;
pvals.bc <- sapply(zs.d.bc, epvals, zs=zs); #p-values wrt bootstrap distribution of max-t statistic 

table.who.d[vars.sub, 4] <- pvals;
table.who.d.bc[vars.sub, 4] <- pvals.bc;


# Table of Difference of Means and SEs

xtable(table.who.d, digits=2);
xtable(table.who.d.bc, digits=2);


rm(PET, PET.w, cdf.lnw.la, cdf.lnw.ha, draws, bse, trim.tails, zs, delta, ucdf.lnw.la, ucdf.lnw.ha, lcdf.lnw.la, lcdf.lnw.ha, 
   ucdf.lnw.la.bc, ucdf.lnw.ha.bc, lcdf.lnw.la.bc, lcdf.lnw.ha.bc);

###################################;
# Part 2: effects on the treated;
###################################;

PET      <- matrix(PE[which(female==1),], ncol = 1, byrow = FALSE);  
PET.w    <- matrix(weight[female==1], ncol = 1, nrow = nrow(PET), byrow = FALSE);

# centered/scaled draws

est.SPET <- wtd.quantile(PET, PET.w, us);   # these are estimated SPEs
draws.SPET<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 1 + c(1:length(us))];  #these are bs draws
delta    <- draws.SPET-matrix(est.SPET, nrow = nrow(draws.SPET), ncol = ncol(draws.SPET), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.SPET), ncol = ncol(draws.SPET), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha, na.rm = TRUE);  #critical value

ubound.SPET<-  sort(est.SPET + crt*sqrt(variance));  #upper conf band
lbound.SPET<-  sort(est.SPET - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.SPET.bc <- sort(2*est.SPET - apply(draws.SPET,2,mean));
ubound.SPET.bc <-  sort(2*est.SPET - apply(draws.SPET,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.SPET.bc <-  sort(2*est.SPET - apply(draws.SPET,2,mean) - crt*sqrt(variance));  #lower conf band


# APE-T;
est.APET  <- weighted.mean(PET, PET.w);
rm(PET, PET.w, d0, d1);
draws.APET<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 1];
delta    <- draws.APET - est.APET;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- mean(delta*delta, na.rm = TRUE);
mzs      <- abs(delta /sqrt(variance));  
crt2     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.APET<- est.APET + crt2*sqrt(variance);  #upper conf bound
lbound.APET<- est.APET - crt2*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.APET.bc  <- 2*est.APET - mean(draws.APET);
ubound.APET.bc <- est.APET.bc + crt2*sqrt(variance);  #upper conf bound
lbound.APET.bc <- est.APET.bc - crt2*sqrt(variance);  #lower conf bound


rm(PET,PET.w,draws.SPET,draws.APET);

# Figure of results;

cairo_pdf("Gender-SPET-qr.pdf", pointsize=15,width=8.0,height=8.0)
plot(us, est.SPET, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     #     main="Women",
     sub="Quantile model", 
     ylim=c(-.7, .3))

polygon(c(us,rev(us)),c(ubound.SPET,rev(lbound.SPET)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.SPET, lwd=2, col = 4 )
abline(h = est.APET, col = 1);
abline(h = ubound.APET, col = 1, lty = 2);
abline(h = lbound.APET, col = 1, lty = 2);
points(c(min(us),.2,.4,.6,.8,max(us)), rep(est.APET,6),  col=1, pch=15 )
legend( x="topleft", 
        legend=c("SPE","APE","90% CB(SPE)", "90% CB(APE)"), 
        col=c(4,1,"light blue",1), lwd=c(1, 1, 5, 1), lty=c(1, 1, 1, 2), pch=c(NA, 15, NA, NA), pt.cex = c(2, 1), bty = 'n' )

dev.off();

cairo_pdf("Gender-SPET-bc-qr.pdf", pointsize=15,width=8.0,height=8.0)
plot(us, est.SPET.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     #     main="Women",
     sub="Quantile model", 
     ylim=c(-.65, .25))

polygon(c(us,rev(us)),c(ubound.SPET.bc,rev(lbound.SPET.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.SPET.bc, lwd=2, col = 4 )
abline(h = est.APET.bc, col = 1);
abline(h = ubound.APET.bc, col = 1, lty = 2);
abline(h = lbound.APET.bc, col = 1, lty = 2);
points(c(min(us),.2,.4,.6,.8,max(us)), rep(est.APET.bc,6),  col=1, pch=15 )
legend( x="topleft", 
        legend=c("SPE","APE","90% CB(SPE)", "90% CB(APE)"), 
        col=c(4,1,"light blue",1), lwd=c(1, 1, 5, 1), lty=c(1, 1, 1, 2), pch=c(NA, 15, NA, NA), pt.cex = c(2, 1), bty = 'n' )

dev.off();

#############################################################;
# Part 3: Conditional effects by education groups for women;
#############################################################;

# No more than HS (lhs, hsg);

hs <- lhs + hsg;
PE.hs <- matrix(PE[hs == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.hs <- matrix(weight[hs==1 & female==1], ncol = 1, nrow = nrow(PE.hs), byrow = FALSE);
est.CSPE1<- wtd.quantile(PE.hs, w.hs, us);   # these are estimated SPEs
rm(hs);
draws.CSPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + length(us) + 2 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE1-matrix(est.CSPE1, nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE1<-  sort(est.CSPE1 + crt*sqrt(variance));  #upper conf band
lbound.CSPE1<-  sort(est.CSPE1 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE1.bc <- sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean));
ubound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) - crt*sqrt(variance));  #lower conf band


# Some College;

PE.sc <- matrix(PE[sc == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.sc <- matrix(weight[sc ==1 & female==1], ncol = 1, nrow = nrow(PE.sc), byrow = FALSE);
est.CSPE2<- wtd.quantile(PE.sc, w.sc, us);   # these are estimated SPEs
draws.CSPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 2*length(us) + 3 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE2-matrix(est.CSPE2, nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE2<-  sort(est.CSPE2 + crt*sqrt(variance));  #upper conf band
lbound.CSPE2<-  sort(est.CSPE2 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE2.bc <- sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean));
ubound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) - crt*sqrt(variance));  #lower conf band


# College graduate;

PE.cg <- matrix(PE[cg == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.cg <- matrix(weight[cg ==1 & female==1], ncol = 1, nrow = nrow(PE.cg), byrow = FALSE);
est.CSPE3<- wtd.quantile(PE.cg, w.cg, us);   # these are estimated SPEs
draws.CSPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 3*length(us) + 4 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE3-matrix(est.CSPE3, nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE3<-  sort(est.CSPE3 + crt*sqrt(variance));  #upper conf band
lbound.CSPE3<-  sort(est.CSPE3 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE3.bc <- sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean));
ubound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) - crt*sqrt(variance));  #lower conf band


# Advanced Degree;

PE.ad <- matrix(PE[ad == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.ad <- matrix(weight[ad==1 & female==1], ncol = 1, nrow = nrow(PE.ad), byrow = FALSE);
est.CSPE4<- wtd.quantile(PE.ad, w.ad, us);   # these are estimated SPEs
draws.CSPE4<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 4*length(us) + 5 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE4-matrix(est.CSPE4, nrow = nrow(draws.CSPE4), ncol = ncol(draws.CSPE4), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE4), ncol = ncol(draws.CSPE4), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE4<-  sort(est.CSPE4 + crt*sqrt(variance));  #upper conf band
lbound.CSPE4<-  sort(est.CSPE4 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE4.bc <- sort(2*est.CSPE4 - apply(draws.CSPE4,2,mean));
ubound.CSPE4.bc <-  sort(2*est.CSPE4 - apply(draws.CSPE4,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE4.bc <-  sort(2*est.CSPE4 - apply(draws.CSPE4,2,mean) - crt*sqrt(variance));  #lower conf band


# APEs;

# Max HS (lhs, hsg);

est.CAPE1  <- weighted.mean(PE.hs, w.hs);
draws.CAPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + length(us) + 2];
delta    <- draws.CAPE1 - est.CAPE1;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt1     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE1<-  sort(est.CAPE1+ crt1*sqrt(variance));  #upper conf bound
lbound.CAPE1<-  sort(est.CAPE1 - crt1*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE1.bc  <- 2*est.CAPE1 - mean(draws.CAPE1);
ubound.CAPE1.bc <- est.CAPE1.bc + crt1*sqrt(variance);  #upper conf bound
lbound.CAPE1.bc <- est.CAPE1.bc - crt1*sqrt(variance);  #lower conf bound


# Some College;

est.CAPE2  <- weighted.mean(PE.sc, w.sc);
draws.CAPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 2*length(us) + 3];
delta    <- draws.CAPE2 - est.CAPE2;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt2     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE2<-  sort(est.CAPE2+ crt2*sqrt(variance));  #upper conf bound
lbound.CAPE2<-  sort(est.CAPE2 - crt2*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE2.bc  <- 2*est.CAPE2 - mean(draws.CAPE2);
ubound.CAPE2.bc <- est.CAPE2.bc + crt2*sqrt(variance);  #upper conf bound
lbound.CAPE2.bc <- est.CAPE2.bc - crt2*sqrt(variance);  #lower conf bound


# College Graduate;

est.CAPE3  <- weighted.mean(PE.cg, w.cg);
draws.CAPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 3*length(us) + 4];
delta    <- draws.CAPE3 - est.CAPE3;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt3     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE3<-  sort(est.CAPE3+ crt3*sqrt(variance));  #upper conf bound
lbound.CAPE3<-  sort(est.CAPE3 - crt3*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE3.bc  <- 2*est.CAPE3 - mean(draws.CAPE3);
ubound.CAPE3.bc <- est.CAPE3.bc + crt3*sqrt(variance);  #upper conf bound
lbound.CAPE3.bc <- est.CAPE3.bc - crt3*sqrt(variance);  #lower conf bound



# Advanced Degree;

est.CAPE4  <- weighted.mean(PE.ad, w.ad);
draws.CAPE4<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 4*length(us) + 5];
delta    <- draws.CAPE4 - est.CAPE4;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt4     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE4<-  sort(est.CAPE4+ crt4*sqrt(variance));  #upper conf bound
lbound.CAPE4<-  sort(est.CAPE4 - crt4*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE4.bc  <- 2*est.CAPE4 - mean(draws.CAPE4);
ubound.CAPE4.bc <- est.CAPE4.bc + crt4*sqrt(variance);  #upper conf bound
lbound.CAPE4.bc <- est.CAPE4.bc - crt4*sqrt(variance);  #lower conf bound


rm(PE.hs, PE.sc,  PE.cg, PE.ad, w.hs, w.sc, w.cg, w.ad, draws.CAPE1, draws.CAPE2, draws.CAPE3, draws.CAPE4, draws.CSPE1, draws.CSPE2, draws.CSPE3, draws.CSPE4);


# Figure of results;

cairo_pdf("Gender-CSPET-ED-qr.pdf", pointsize=15,width=8.0,height=8.0);

par(mfrow=c(2,2));

plot(us, est.CSPE1, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="HS Degree or less",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1,rev(lbound.CSPE1)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1, lwd=2, col = 4 )
abline(h = est.CAPE1, col = 1);
abline(h = ubound.CAPE1, col = 1, lty = 2);
abline(h = lbound.CAPE1, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Some College",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2,rev(lbound.CSPE2)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2, lwd=2, col = 4 )
abline(h = est.CAPE2, col = 1);
abline(h = ubound.CAPE2, col = 1, lty = 2);
abline(h = lbound.CAPE2, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="College Graduate",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE3,rev(lbound.CSPE3)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3, lwd=2, col = 4 )
abline(h = est.CAPE3, col = 1);
abline(h = ubound.CAPE3, col = 1, lty = 2);
abline(h = lbound.CAPE3, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE4, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Advanced Degree",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE4,rev(lbound.CSPE4)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE4, lwd=2, col = 4 )
abline(h = est.CAPE4, col = 1);
abline(h = ubound.CAPE4, col = 1, lty = 2);
abline(h = lbound.CAPE4, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

dev.off();


cairo_pdf("Gender-CSPET-ED-bc-qr.pdf", pointsize=15,width=8.0,height=8.0);

par(mfrow=c(2,2));

plot(us, est.CSPE1.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="HS Degree or less",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1.bc,rev(lbound.CSPE1.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1.bc, lwd=2, col = 4 )
abline(h = est.CAPE1.bc, col = 1);
abline(h = ubound.CAPE1.bc, col = 1, lty = 2);
abline(h = lbound.CAPE1.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Some College",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2.bc,rev(lbound.CSPE2.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2.bc, lwd=2, col = 4 )
abline(h = est.CAPE2.bc, col = 1);
abline(h = ubound.CAPE2.bc, col = 1, lty = 2);
abline(h = lbound.CAPE2.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="College Graduate",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE3.bc,rev(lbound.CSPE3.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3.bc, lwd=2, col = 4 )
abline(h = est.CAPE3.bc, col = 1);
abline(h = ubound.CAPE3.bc, col = 1, lty = 2);
abline(h = lbound.CAPE3.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE4.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Advanced Degree",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE4.bc,rev(lbound.CSPE4.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE4.bc, lwd=2, col = 4 )
abline(h = est.CAPE4.bc, col = 1);
abline(h = ubound.CAPE4.bc, col = 1, lty = 2);
abline(h = lbound.CAPE4.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

dev.off();




############################################################;
# Part 4: Conditional effects by marital status for women;
############################################################;


# Never Married;

nm<- nevermarried;
PE.nm <- matrix(PE[nm == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.nm <- matrix(weight[nm==1 & female==1], ncol = 1, nrow = nrow(PE.nm), byrow = FALSE);
est.CSPE1<- wtd.quantile(PE.nm, w.nm, us);   # these are estimated SPEs
rm(nm);
draws.CSPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 5*length(us) + 6 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE1-matrix(est.CSPE1, nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE1<-  sort(est.CSPE1 + crt*sqrt(variance));  #upper conf band
lbound.CSPE1<-  sort(est.CSPE1 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE1.bc <- sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean));
ubound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) - crt*sqrt(variance));  #lower conf band


# Married;

ma  <- married;
PE.ma <- matrix(PE[ma == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.ma <- matrix(weight[ma==1 & female==1], ncol = 1, nrow = nrow(PE.ma), byrow = FALSE);
est.CSPE2<- wtd.quantile(PE.ma, w.ma, us);   # these are estimated SPEs
rm(ma);
draws.CSPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 6*length(us) + 7 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE2-matrix(est.CSPE2, nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE2<-  sort(est.CSPE2 + crt*sqrt(variance));  #upper conf band
lbound.CSPE2<-  sort(est.CSPE2 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE2.bc <- sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean));
ubound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) - crt*sqrt(variance));  #lower conf band


# Divorced;

di <- divorced;
PE.di <- matrix(PE[di == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.di <- matrix(weight[di==1 & female==1], ncol = 1, nrow = nrow(PE.di), byrow = FALSE);
est.CSPE3<- wtd.quantile(PE.di, w.di, us);   # these are estimated SPEs
rm(di);
draws.CSPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 7*length(us) + 8 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE3-matrix(est.CSPE3, nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE3<-  sort(est.CSPE3 + crt*sqrt(variance));  #upper conf band
lbound.CSPE3<-  sort(est.CSPE3 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE3.bc <- sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean));
ubound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) - crt*sqrt(variance));  #lower conf band


# APEs;

# Never Married;

est.CAPE1  <- weighted.mean(PE.nm, w.nm);
draws.CAPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 5*length(us) + 6];
delta    <- draws.CAPE1 - est.CAPE1;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt1     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE1<-  sort(est.CAPE1+ crt1*sqrt(variance));  #upper conf bound
lbound.CAPE1<-  sort(est.CAPE1 - crt1*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE1.bc  <- 2*est.CAPE1 - mean(draws.CAPE1);
ubound.CAPE1.bc <- est.CAPE1.bc + crt1*sqrt(variance);  #upper conf bound
lbound.CAPE1.bc <- est.CAPE1.bc - crt1*sqrt(variance);  #lower conf bound


# Married;


est.CAPE2  <- weighted.mean(PE.ma, w.ma);
draws.CAPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 6*length(us) + 7];
delta    <- draws.CAPE2 - est.CAPE2;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt2     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE2<-  sort(est.CAPE2+ crt2*sqrt(variance));  #upper conf bound
lbound.CAPE2<-  sort(est.CAPE2 - crt2*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE2.bc  <- 2*est.CAPE2 - mean(draws.CAPE2);
ubound.CAPE2.bc <- est.CAPE2.bc + crt2*sqrt(variance);  #upper conf bound
lbound.CAPE2.bc <- est.CAPE2.bc - crt2*sqrt(variance);  #lower conf bound


# Divorced;

est.CAPE3  <- weighted.mean(PE.di, w.di);
draws.CAPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 7*length(us) + 8];
delta    <- draws.CAPE3 - est.CAPE3;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt3     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE3<-  sort(est.CAPE3+ crt3*sqrt(variance));  #upper conf bound
lbound.CAPE3<-  sort(est.CAPE3 - crt3*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE3.bc  <- 2*est.CAPE3 - mean(draws.CAPE3);
ubound.CAPE3.bc <- est.CAPE3.bc + crt3*sqrt(variance);  #upper conf bound
lbound.CAPE3.bc <- est.CAPE3.bc - crt3*sqrt(variance);  #lower conf bound


rm(PE.ma, PE.nm,  PE.di, w.nm, w.ma, w.di, draws.CAPE1, draws.CAPE2, draws.CAPE3, draws.CSPE1, draws.CSPE2, draws.CSPE3);


# Figure of results;

cairo_pdf("Gender-CSPET-MS-qr.pdf", pointsize=15,width=12.0,height=6.0);

par(mfrow=c(1,2));

plot(us, est.CSPE1, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Never Married",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1,rev(lbound.CSPE1)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1, lwd=2, col = 4 )
abline(h = est.CAPE1, col = 1);
abline(h = ubound.CAPE1, col = 1, lty = 2);
abline(h = lbound.CAPE1, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Married",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2,rev(lbound.CSPE2)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2, lwd=2, col = 4 )
abline(h = est.CAPE2, col = 1);
abline(h = ubound.CAPE2, col = 1, lty = 2);
abline(h = lbound.CAPE2, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();

cairo_pdf("Gender-CSPET-MS-bc-qr.pdf", pointsize=15,width=12.0,height=6.0);

par(mfrow=c(1,2));

plot(us, est.CSPE1.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Never Married",
     sub="Quantile model", 
     ylim=c(-.7,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1.bc,rev(lbound.CSPE1.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1.bc, lwd=2, col = 4 )
abline(h = est.CAPE1.bc, col = 1);
abline(h = ubound.CAPE1.bc, col = 1, lty = 2);
abline(h = lbound.CAPE1.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Married",
     sub="Quantile model", 
     ylim=c(-.7,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2.bc,rev(lbound.CSPE2.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2.bc, lwd=2, col = 4 )
abline(h = est.CAPE2.bc, col = 1);
abline(h = ubound.CAPE2.bc, col = 1, lty = 2);
abline(h = lbound.CAPE2.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();


########################################################;
# Part 5: Conditional effects by experience for women;
########################################################;

le <- exp1 <= wtd.quantile(exp1[female==1], weight[female==1], 1/4);
me <- exp1 > wtd.quantile(exp1[female==1], weight[female==1], 1/4) & exp1 < wtd.quantile(exp1[female==1], weight[female==1], 3/4);
he <- exp1 >= wtd.quantile(exp1[female==1], weight[female==1], 3/4);

# Low-Experience;

PE.le <- matrix(PE[le == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.le <- matrix(weight[le==1 & female==1], ncol = 1, nrow = nrow(PE.le), byrow = FALSE);
est.CSPE1<- wtd.quantile(PE.le, w.le, us);   # these are estimated SPEs
rm(le);
draws.CSPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 8*length(us) + 9 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE1-matrix(est.CSPE1, nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE1<-  sort(est.CSPE1 + crt*sqrt(variance));  #upper conf band
lbound.CSPE1<-  sort(est.CSPE1 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE1.bc <- sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean));
ubound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) - crt*sqrt(variance));  #lower conf band


# Mid-Experience;

PE.me <- matrix(PE[me == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.me <- matrix(weight[me==1 & female==1], ncol = 1, nrow = nrow(PE.me), byrow = FALSE);
est.CSPE2<- wtd.quantile(PE.me, w.me, us);   # these are estimated SPEs
rm(me);
draws.CSPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 9*length(us) + 10 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE2-matrix(est.CSPE2, nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE2<-  sort(est.CSPE2 + crt*sqrt(variance));  #upper conf band
lbound.CSPE2<-  sort(est.CSPE2 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE2.bc <- sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean));
ubound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) - crt*sqrt(variance));  #lower conf band


# High-Experience;

PE.he <- matrix(PE[he == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.he <- matrix(weight[he==1 & female==1], ncol = 1, nrow = nrow(PE.he), byrow = FALSE);
est.CSPE3<- wtd.quantile(PE.he, w.he, us);   # these are estimated SPEs
rm(he);
draws.CSPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 10*length(us) + 11 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE3-matrix(est.CSPE3, nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE3<-  sort(est.CSPE3 + crt*sqrt(variance));  #upper conf band
lbound.CSPE3<-  sort(est.CSPE3 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE3.bc <- sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean));
ubound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) - crt*sqrt(variance));  #lower conf band


# APEs;

# Low-Experience;

est.CAPE1  <- weighted.mean(PE.le, w.le);
draws.CAPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 8*length(us) + 9];
delta    <- draws.CAPE1 - est.CAPE1;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt1     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE1<-  sort(est.CAPE1+ crt1*sqrt(variance));  #upper conf bound
lbound.CAPE1<-  sort(est.CAPE1 - crt1*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE1.bc  <- 2*est.CAPE1 - mean(draws.CAPE1);
ubound.CAPE1.bc <- est.CAPE1.bc + crt1*sqrt(variance);  #upper conf bound
lbound.CAPE1.bc <- est.CAPE1.bc - crt1*sqrt(variance);  #lower conf bound


# Mid-Experience;


est.CAPE2  <- weighted.mean(PE.me, w.me);
draws.CAPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 9*length(us) + 10];
delta    <- draws.CAPE2 - est.CAPE2;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt2     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE2<-  sort(est.CAPE2+ crt2*sqrt(variance));  #upper conf bound
lbound.CAPE2<-  sort(est.CAPE2 - crt2*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE2.bc  <- 2*est.CAPE2 - mean(draws.CAPE2);
ubound.CAPE2.bc <- est.CAPE2.bc + crt2*sqrt(variance);  #upper conf bound
lbound.CAPE2.bc <- est.CAPE2.bc - crt2*sqrt(variance);  #lower conf bound


# High-Experience;

est.CAPE3  <- weighted.mean(PE.he, w.he);
draws.CAPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 10*length(us) + 11];
delta    <- draws.CAPE3 - est.CAPE3;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt3     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE3<-  sort(est.CAPE3+ crt3*sqrt(variance));  #upper conf bound
lbound.CAPE3<-  sort(est.CAPE3 - crt3*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE3.bc  <- 2*est.CAPE3 - mean(draws.CAPE3);
ubound.CAPE3.bc <- est.CAPE3.bc + crt3*sqrt(variance);  #upper conf bound
lbound.CAPE3.bc <- est.CAPE3.bc - crt3*sqrt(variance);  #lower conf bound


rm(PE.le, PE.me,  PE.he, w.le, w.me, w.he, draws.CAPE1, draws.CAPE2, draws.CAPE3, draws.CSPE1, draws.CSPE2, draws.CSPE3);


# Figure of results;

cairo_pdf("Gender-CSPET-EXP-qr.pdf", pointsize=15,width=8.0,height=4.0);

par(mfrow=c(1,3));

plot(us, est.CSPE1, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Low-Experience",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1,rev(lbound.CSPE1)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1, lwd=2, col = 4 )
abline(h = est.CAPE1, col = 1);
abline(h = ubound.CAPE1, col = 1, lty = 2);
abline(h = lbound.CAPE1, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Mid-Experience",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2,rev(lbound.CSPE2)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2, lwd=2, col = 4 )
abline(h = est.CAPE2, col = 1);
abline(h = ubound.CAPE2, col = 1, lty = 2);
abline(h = lbound.CAPE2, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="High-Experience",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE3,rev(lbound.CSPE3)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3, lwd=2, col = 4 )
abline(h = est.CAPE3, col = 1);
abline(h = ubound.CAPE3, col = 1, lty = 2);
abline(h = lbound.CAPE3, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();


cairo_pdf("Gender-CSPET-EXP-bc-qr.pdf", pointsize=15,width=8.0,height=4.0);

par(mfrow=c(1,3));

plot(us, est.CSPE1.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Low-Experience",
     sub="Quantile model", 
     ylim=c(-.7,.3))
polygon(c(us,rev(us)),c(ubound.CSPE1.bc,rev(lbound.CSPE1.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1.bc, lwd=2, col = 4 )
abline(h = est.CAPE1.bc, col = 1);
abline(h = ubound.CAPE1.bc, col = 1, lty = 2);
abline(h = lbound.CAPE1.bc, col = 1, lty = 2);
legend(0, .3, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .3, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Mid-Experience",
     sub="Quantile model", 
     ylim=c(-.7,.3))
polygon(c(us,rev(us)),c(ubound.CSPE2.bc,rev(lbound.CSPE2.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2.bc, lwd=2, col = 4 )
abline(h = est.CAPE2.bc, col = 1);
abline(h = ubound.CAPE2.bc, col = 1, lty = 2);
abline(h = lbound.CAPE2.bc, col = 1, lty = 2);
legend(0, .3, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .3, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="High-Experience",
     sub="Quantile model", 
     ylim=c(-.7,.3))
polygon(c(us,rev(us)),c(ubound.CSPE3.bc,rev(lbound.CSPE3.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3.bc, lwd=2, col = 4 )
abline(h = est.CAPE3.bc, col = 1);
abline(h = ubound.CAPE3.bc, col = 1, lty = 2);
abline(h = lbound.CAPE3.bc, col = 1, lty = 2);
legend(0, .3, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .3, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();


##########################################################;
# Part 6: Conditional effects by occupation for women;
##########################################################;


# Manager;

ma <- occ.manager;
PE.ma <- matrix(PE[ma == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.ma <- matrix(weight[ma==1 & female==1], ncol = 1, nrow = nrow(PE.ma), byrow = FALSE);
est.CSPE1<- wtd.quantile(PE.ma, w.ma, us);   # these are estimated SPEs
rm(ma);
draws.CSPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 11*length(us) + 12 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE1-matrix(est.CSPE1, nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE1<-  sort(est.CSPE1 + crt*sqrt(variance));  #upper conf band
lbound.CSPE1<-  sort(est.CSPE1 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE1.bc <- sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean));
ubound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) - crt*sqrt(variance));  #lower conf band



# Sales;

sa <- occ.sales;
PE.sa <- matrix(PE[sa == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.sa <- matrix(weight[sa==1 & female==1], ncol = 1, nrow = nrow(PE.sa), byrow = FALSE);
est.CSPE2<- wtd.quantile(PE.sa, w.sa, us);   # these are estimated SPEs
rm(sa);
draws.CSPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 12*length(us) + 13 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE2-matrix(est.CSPE2, nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE2<-  sort(est.CSPE2 + crt*sqrt(variance));  #upper conf band
lbound.CSPE2<-  sort(est.CSPE2 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE2.bc <- sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean));
ubound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) - crt*sqrt(variance));  #lower conf band



# Service;

se <- occ.service;
PE.se <- matrix(PE[se == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.se <- matrix(weight[se==1 & female==1], ncol = 1, nrow = nrow(PE.se), byrow = FALSE);
est.CSPE3<- wtd.quantile(PE.se, w.se, us);   # these are estimated SPEs
rm(se);
draws.CSPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 13*length(us) + 14 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE3-matrix(est.CSPE3, nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE3<-  sort(est.CSPE3 + crt*sqrt(variance));  #upper conf band
lbound.CSPE3<-  sort(est.CSPE3 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE3.bc <- sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean));
ubound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) - crt*sqrt(variance));  #lower conf band



# Production;

pr <- occ.production;
PE.pr <- matrix(PE[pr == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.pr <- matrix(weight[pr==1 & female==1], ncol = 1, nrow = nrow(PE.pr), byrow = FALSE);
est.CSPE4<- wtd.quantile(PE.pr, w.pr, us);   # these are estimated SPEs
rm(pr);
draws.CSPE4<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 14*length(us) + 15 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE4-matrix(est.CSPE4, nrow = nrow(draws.CSPE4), ncol = ncol(draws.CSPE4), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE4), ncol = ncol(draws.CSPE4), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE4<-  sort(est.CSPE4 + crt*sqrt(variance));  #upper conf band
lbound.CSPE4<-  sort(est.CSPE4 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE4.bc <- sort(2*est.CSPE4 - apply(draws.CSPE4,2,mean));
ubound.CSPE4.bc <-  sort(2*est.CSPE4 - apply(draws.CSPE4,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE4.bc <-  sort(2*est.CSPE4 - apply(draws.CSPE4,2,mean) - crt*sqrt(variance));  #lower conf band



# APEs;

# Manager;

est.CAPE1  <- weighted.mean(PE.ma, w.ma);
draws.CAPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 11*length(us) + 12];
delta    <- draws.CAPE1 - est.CAPE1;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt1     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE1<-  est.CAPE1+ crt1*sqrt(variance);  #upper conf bound
lbound.CAPE1<- est.CAPE1 - crt1*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE1.bc  <- 2*est.CAPE1 - mean(draws.CAPE1);
ubound.CAPE1.bc <- est.CAPE1.bc + crt1*sqrt(variance);  #upper conf bound
lbound.CAPE1.bc <- est.CAPE1.bc - crt1*sqrt(variance);  #lower conf bound


# Sales;

est.CAPE2  <- weighted.mean(PE.sa, w.sa);
draws.CAPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 12*length(us) + 13];
delta    <- draws.CAPE2 - est.CAPE2;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt2     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE2<-  est.CAPE2+ crt2*sqrt(variance);  #upper conf bound
lbound.CAPE2<- est.CAPE2 - crt2*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE2.bc  <- 2*est.CAPE2 - mean(draws.CAPE2);
ubound.CAPE2.bc <- est.CAPE2.bc + crt2*sqrt(variance);  #upper conf bound
lbound.CAPE2.bc <- est.CAPE2.bc - crt2*sqrt(variance);  #lower conf bound


# Services;

est.CAPE3  <- weighted.mean(PE.se, w.se);
draws.CAPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 13*length(us) + 14];
delta    <- draws.CAPE3 - est.CAPE3;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt3     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE3<-  est.CAPE3+ crt3*sqrt(variance);  #upper conf bound
lbound.CAPE3<- est.CAPE3 - crt3*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE3.bc  <- 2*est.CAPE3 - mean(draws.CAPE3);
ubound.CAPE3.bc <- est.CAPE3.bc + crt3*sqrt(variance);  #upper conf bound
lbound.CAPE3.bc <- est.CAPE3.bc - crt3*sqrt(variance);  #lower conf bound


# Production;

est.CAPE4  <- weighted.mean(PE.pr, w.pr);
draws.CAPE4<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 14*length(us) + 15];
delta    <- draws.CAPE4 - est.CAPE4;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt4     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE4<-  est.CAPE4+ crt4*sqrt(variance);  #upper conf bound
lbound.CAPE4<- est.CAPE4 - crt4*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE4.bc  <- 2*est.CAPE4 - mean(draws.CAPE4);
ubound.CAPE4.bc <- est.CAPE4.bc + crt4*sqrt(variance);  #upper conf bound
lbound.CAPE4.bc <- est.CAPE4.bc - crt4*sqrt(variance);  #lower conf bound


rm(PE.ma, PE.sa,  PE.se, PE.pr,  w.ma, w.sa, w.se, w.pr, draws.CAPE1, draws.CAPE2, draws.CAPE3, draws.CAPE4, draws.CSPE4, draws.CSPE1, draws.CSPE2, draws.CSPE3);

# Figure of results;

cairo_pdf("Gender-CSPET-OC-qr-full.pdf", pointsize=15,width=8.0,height=8.0);

par(mfrow=c(2,2));

plot(us, est.CSPE1, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Manager",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1,rev(lbound.CSPE1)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1, lwd=2, col = 4 )
abline(h = est.CAPE1, col = 1);
abline(h = ubound.CAPE1, col = 1, lty = 2);
abline(h = lbound.CAPE1, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Sales",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2,rev(lbound.CSPE2)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2, lwd=2, col = 4 )
abline(h = est.CAPE2, col = 1);
abline(h = ubound.CAPE2, col = 1, lty = 2);
abline(h = lbound.CAPE2, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Service",
     sub="Quantile model",
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE3,rev(lbound.CSPE3)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3, lwd=2, col = 4 )
abline(h = est.CAPE3, col = 1);
abline(h = ubound.CAPE3, col = 1, lty = 2);
abline(h = lbound.CAPE3, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE4, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Production",
     sub="Quantile model",
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE4,rev(lbound.CSPE4)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE4, lwd=2, col = 4 )
abline(h = est.CAPE4, col = 1);
abline(h = ubound.CAPE4, col = 1, lty = 2);
abline(h = lbound.CAPE4, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();

cairo_pdf("Gender-CSPET-OC-bc-qr-full.pdf", pointsize=15,width=8.0,height=8.0);

par(mfrow=c(2,2));

plot(us, est.CSPE1.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Manager",
     sub="Quantile model", 
     ylim=c(-.75,.6))
polygon(c(us,rev(us)),c(ubound.CSPE1.bc,rev(lbound.CSPE1.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1.bc, lwd=2, col = 4 )
abline(h = est.CAPE1.bc, col = 1);
abline(h = ubound.CAPE1.bc, col = 1, lty = 2);
abline(h = lbound.CAPE1.bc, col = 1, lty = 2);
legend(0, .6, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .6, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Sales",
     sub="Quantile model", 
     ylim=c(-.75,.6))
polygon(c(us,rev(us)),c(ubound.CSPE2.bc,rev(lbound.CSPE2.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2.bc, lwd=2, col = 4 )
abline(h = est.CAPE2.bc, col = 1);
abline(h = ubound.CAPE2.bc, col = 1, lty = 2);
abline(h = lbound.CAPE2.bc, col = 1, lty = 2);
legend(0, .6, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .6, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Service",
     sub="Quantile model",
     ylim=c(-.75,.6))
polygon(c(us,rev(us)),c(ubound.CSPE3.bc,rev(lbound.CSPE3.bc)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3.bc, lwd=2, col = 4 )
abline(h = est.CAPE3.bc, col = 1);
abline(h = ubound.CAPE3.bc, col = 1, lty = 2);
abline(h = lbound.CAPE3.bc, col = 1, lty = 2);
legend(0, .6, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .6, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE4.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Production",
     sub="Quantile model",
     ylim=c(-.75,.6))
polygon(c(us,rev(us)),c(ubound.CSPE4.bc,rev(lbound.CSPE4.bc)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE4.bc, lwd=2, col = 4 )
abline(h = est.CAPE4.bc, col = 1);
abline(h = ubound.CAPE4.bc, col = 1, lty = 2);
abline(h = lbound.CAPE4.bc, col = 1, lty = 2);
legend(0, .6, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .6, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();

#####################################################;
# Part 7: Conditional effects by industry for women;
#####################################################;

# Manufacture;

ma <- ind.manufacture;
PE.ma <- matrix(PE[ma == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.ma <- matrix(weight[ma==1 & female==1], ncol = 1, nrow = nrow(PE.ma), byrow = FALSE);
est.CSPE1<- wtd.quantile(PE.ma, w.ma, us);   # these are estimated SPEs
rm(ma);
draws.CSPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 15*length(us) + 16 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE1-matrix(est.CSPE1, nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE1<-  sort(est.CSPE1 + crt*sqrt(variance));  #upper conf band
lbound.CSPE1<-  sort(est.CSPE1 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE1.bc <- sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean));
ubound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) - crt*sqrt(variance));  #lower conf band



# Finance;

fi <- ind.finance;
PE.fi <- matrix(PE[fi == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.fi <- matrix(weight[fi==1 & female==1], ncol = 1, nrow = nrow(PE.fi), byrow = FALSE);
est.CSPE2<- wtd.quantile(PE.fi, w.fi, us);   # these are estimated SPEs
rm(fi);
draws.CSPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 16*length(us) + 17 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE2-matrix(est.CSPE2, nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE2<-  sort(est.CSPE2 + crt*sqrt(variance));  #upper conf band
lbound.CSPE2<-  sort(est.CSPE2 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE2.bc <- sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean));
ubound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) - crt*sqrt(variance));  #lower conf band



# Professional;

pr <- ind.professional;
PE.pr <- matrix(PE[pr == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.pr <- matrix(weight[pr==1 & female==1], ncol = 1, nrow = nrow(PE.pr), byrow = FALSE);
est.CSPE3<- wtd.quantile(PE.pr, w.pr, us);   # these are estimated SPEs
rm(pr);
draws.CSPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 17*length(us) + 18 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE3-matrix(est.CSPE3, nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE3<-  sort(est.CSPE3 + crt*sqrt(variance));  #upper conf band
lbound.CSPE3<-  sort(est.CSPE3 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE3.bc <- sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean));
ubound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) - crt*sqrt(variance));  #lower conf band



# Education;

ed <- ind.education;
PE.ed <- matrix(PE[ed == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.ed <- matrix(weight[ed==1 & female==1], ncol = 1, nrow = nrow(PE.ed), byrow = FALSE);
est.CSPE4<- wtd.quantile(PE.ed, w.ed, us);   # these are estimated SPEs
rm(ed);
draws.CSPE4<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 18*length(us) + 19 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE4-matrix(est.CSPE4, nrow = nrow(draws.CSPE4), ncol = ncol(draws.CSPE4), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE4), ncol = ncol(draws.CSPE4), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE4<-  sort(est.CSPE4 + crt*sqrt(variance));  #upper conf band
lbound.CSPE4<-  sort(est.CSPE4 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE4.bc <- sort(2*est.CSPE4 - apply(draws.CSPE4,2,mean));
ubound.CSPE4.bc <-  sort(2*est.CSPE4 - apply(draws.CSPE4,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE4.bc <-  sort(2*est.CSPE4 - apply(draws.CSPE4,2,mean) - crt*sqrt(variance));  #lower conf band




# leisure;

le <- ind.leisure;
PE.le <- matrix(PE[le == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.le <- matrix(weight[le==1 & female==1], ncol = 1, nrow = nrow(PE.le), byrow = FALSE);
est.CSPE5<- wtd.quantile(PE.le, w.le, us);   # these are estimated SPEs
rm(le);
draws.CSPE5<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 19*length(us) + 20 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE5-matrix(est.CSPE5, nrow = nrow(draws.CSPE5), ncol = ncol(draws.CSPE5), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE5), ncol = ncol(draws.CSPE5), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE5<-  sort(est.CSPE5 + crt*sqrt(variance));  #upper conf band
lbound.CSPE5<-  sort(est.CSPE5 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE5.bc <- sort(2*est.CSPE5 - apply(draws.CSPE5,2,mean));
ubound.CSPE5.bc <-  sort(2*est.CSPE5 - apply(draws.CSPE5,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE5.bc <-  sort(2*est.CSPE5 - apply(draws.CSPE5,2,mean) - crt*sqrt(variance));  #lower conf band




# public;

pu <- ind.public;
PE.pu <- matrix(PE[pu == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.pu <- matrix(weight[pu==1 & female==1], ncol = 1, nrow = nrow(PE.pu), byrow = FALSE);
est.CSPE6<- wtd.quantile(PE.pu, w.pu, us);   # these are estimated SPEs
rm(pu);
draws.CSPE6<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 20*length(us) + 21 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE6-matrix(est.CSPE6, nrow = nrow(draws.CSPE6), ncol = ncol(draws.CSPE6), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE6), ncol = ncol(draws.CSPE6), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE6<-  sort(est.CSPE6 + crt*sqrt(variance));  #upper conf band
lbound.CSPE6<-  sort(est.CSPE6 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE6.bc <- sort(2*est.CSPE6 - apply(draws.CSPE6,2,mean));
ubound.CSPE6.bc <-  sort(2*est.CSPE6 - apply(draws.CSPE6,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE6.bc <-  sort(2*est.CSPE6 - apply(draws.CSPE6,2,mean) - crt*sqrt(variance));  #lower conf band




# services;

se <- ind.services;
PE.se <- matrix(PE[se == 1 & female==1, ], ncol = 1, byrow = FALSE);
w.se <- matrix(weight[se==1 & female==1], ncol = 1, nrow = nrow(PE.se), byrow = FALSE);
est.CSPE7<- wtd.quantile(PE.se, w.se, us);   # these are estimated SPEs
rm(se);
draws.CSPE7<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 21*length(us) + 22 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE7-matrix(est.CSPE7, nrow = nrow(draws.CSPE7), ncol = ncol(draws.CSPE7), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
#variance <- apply(delta*delta,2,mean, na.rm = TRUE);
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE7), ncol = ncol(draws.CSPE7), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE7<-  sort(est.CSPE7 + crt*sqrt(variance));  #upper conf band
lbound.CSPE7<-  sort(est.CSPE7 - crt*sqrt(variance));  #lower conf band


# bootstrap bias correction;

est.CSPE7.bc <- sort(2*est.CSPE7 - apply(draws.CSPE7,2,mean));
ubound.CSPE7.bc <-  sort(2*est.CSPE7 - apply(draws.CSPE7,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE7.bc <-  sort(2*est.CSPE7 - apply(draws.CSPE7,2,mean) - crt*sqrt(variance));  #lower conf band




# APEs;

# Manufacture;


est.CAPE1  <- weighted.mean(PE.ma, w.ma);
draws.CAPE1<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 15*length(us) + 16];
delta    <- draws.CAPE1 - est.CAPE1;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt1     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE1<-  est.CAPE1+ crt1*sqrt(variance);  #upper conf bound
lbound.CAPE1<- est.CAPE1 - crt1*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE1.bc  <- 2*est.CAPE1 - mean(draws.CAPE1);
ubound.CAPE1.bc <- est.CAPE1.bc + crt1*sqrt(variance);  #upper conf bound
lbound.CAPE1.bc <- est.CAPE1.bc - crt1*sqrt(variance);  #lower conf bound



# Finance;


est.CAPE2  <- weighted.mean(PE.fi, w.fi);
draws.CAPE2<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 16*length(us) + 17];
delta    <- draws.CAPE2 - est.CAPE2;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt2     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE2<-  est.CAPE2+ crt2*sqrt(variance);  #upper conf bound
lbound.CAPE2<- est.CAPE2 - crt2*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE2.bc  <- 2*est.CAPE2 - mean(draws.CAPE2);
ubound.CAPE2.bc <- est.CAPE2.bc + crt2*sqrt(variance);  #upper conf bound
lbound.CAPE2.bc <- est.CAPE2.bc - crt2*sqrt(variance);  #lower conf bound



# Professional;

est.CAPE3  <- weighted.mean(PE.pr, w.pr);
draws.CAPE3<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 17*length(us) + 18];
delta    <- draws.CAPE3 - est.CAPE3;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt3     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE3<-  est.CAPE3+ crt3*sqrt(variance);  #upper conf bound
lbound.CAPE3<- est.CAPE3 - crt3*sqrt(variance);  #lower conf bound


# bootstrap bias correction;

est.CAPE3.bc  <- 2*est.CAPE3 - mean(draws.CAPE3);
ubound.CAPE3.bc <- est.CAPE3.bc + crt3*sqrt(variance);  #upper conf bound
lbound.CAPE3.bc <- est.CAPE3.bc - crt3*sqrt(variance);  #lower conf bound



# Education;

est.CAPE4  <- weighted.mean(PE.ed, w.ed);
draws.CAPE4<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 18*length(us) + 19];
delta    <- draws.CAPE4 - est.CAPE4;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt4     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE4<-  est.CAPE4+ crt4*sqrt(variance);  #upper conf bound
lbound.CAPE4<- est.CAPE4 - crt4*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE4.bc  <- 2*est.CAPE4 - mean(draws.CAPE4);
ubound.CAPE4.bc <- est.CAPE4.bc + crt4*sqrt(variance);  #upper conf bound
lbound.CAPE4.bc <- est.CAPE4.bc - crt4*sqrt(variance);  #lower conf bound



# leisure;

est.CAPE5  <- weighted.mean(PE.le, w.le);
draws.CAPE5<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 19*length(us) + 20];
delta    <- draws.CAPE5 - est.CAPE5;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt5     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE5<-  est.CAPE5+ crt5*sqrt(variance);  #upper conf bound
lbound.CAPE5<- est.CAPE5 - crt5*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE5.bc  <- 2*est.CAPE5 - mean(draws.CAPE5);
ubound.CAPE5.bc <- est.CAPE5.bc + crt5*sqrt(variance);  #upper conf bound
lbound.CAPE5.bc <- est.CAPE5.bc - crt5*sqrt(variance);  #lower conf bound



# public;

est.CAPE6  <- weighted.mean(PE.pu, w.pu);
draws.CAPE6<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 20*length(us) + 21];
delta    <- draws.CAPE6 - est.CAPE6;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt6     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE6<-  est.CAPE6+ crt6*sqrt(variance);  #upper conf bound
lbound.CAPE6<- est.CAPE6 - crt6*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE6.bc  <- 2*est.CAPE6 - mean(draws.CAPE6);
ubound.CAPE6.bc <- est.CAPE6.bc + crt6*sqrt(variance);  #upper conf bound
lbound.CAPE6.bc <- est.CAPE6.bc - crt6*sqrt(variance);  #lower conf bound



# services;

est.CAPE7  <- weighted.mean(PE.se, w.se);
draws.CAPE7<- result.boot.SE.qr$t[,4*length(vars) + 2*length(lexper) + 2*length(lwage) + 21*length(us) + 22];
delta    <- draws.CAPE7 - est.CAPE7;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt7     <- quantile(mzs, 1-alpha);  #critical value

ubound.CAPE7<-  est.CAPE7+ crt7*sqrt(variance);  #upper conf bound
lbound.CAPE7<- est.CAPE7 - crt7*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.CAPE7.bc  <- 2*est.CAPE7 - mean(draws.CAPE7);
ubound.CAPE7.bc <- est.CAPE7.bc + crt7*sqrt(variance);  #upper conf bound
lbound.CAPE7.bc <- est.CAPE7.bc - crt7*sqrt(variance);  #lower conf bound



rm(PE.ma, PE.fi,  PE.pr, PE.ed, PE.le,  PE.pu, PE.se, w.ma, w.fi,  w.pr, w.ed, w.le,  w.pu, w.se, 
   draws.CAPE1, draws.CAPE2, draws.CAPE3, draws.CAPE4, draws.CAPE5, draws.CAPE6, draws.CAPE7, 
   draws.CSPE1, draws.CSPE2, draws.CSPE3, draws.CSPE4, draws.CSPE5, draws.CSPE6, draws.CSPE7);

# Figure of results;

cairo_pdf("Gender-CSPET-IN-qr-full.pdf", pointsize=15,width=12.0,height=8.0);

par(mfrow=c(2,3));

plot(us, est.CSPE1, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Manufacture",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1,rev(lbound.CSPE1)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1, lwd=2, col = 4 )
abline(h = est.CAPE1, col = 1);
abline(h = ubound.CAPE1, col = 1, lty = 2);
abline(h = lbound.CAPE1, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Finance",
     sub="Quantile model", 
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2,rev(lbound.CSPE2)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2, lwd=2, col = 4 )
abline(h = est.CAPE2, col = 1);
abline(h = ubound.CAPE2, col = 1, lty = 2);
abline(h = lbound.CAPE2, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Professional",
     sub="Quantile model",
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE3,rev(lbound.CSPE3)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3, lwd=2, col = 4 )
abline(h = est.CAPE3, col = 1);
abline(h = ubound.CAPE3, col = 1, lty = 2);
abline(h = lbound.CAPE3, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE4, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Education",
     sub="Quantile model",
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE4,rev(lbound.CSPE4)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE4, lwd=2, col = 4 )
abline(h = est.CAPE4, col = 1);
abline(h = ubound.CAPE4, col = 1, lty = 2);
abline(h = lbound.CAPE4, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE5, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Leisure",
     sub="Quantile model",
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE5,rev(lbound.CSPE5)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE5, lwd=2, col = 4 )
abline(h = est.CAPE5, col = 1);
abline(h = ubound.CAPE5, col = 1, lty = 2);
abline(h = lbound.CAPE5, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE6, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Public",
     sub="Quantile model",
     ylim=c(-.8,.4))
polygon(c(us,rev(us)),c(ubound.CSPE6,rev(lbound.CSPE6)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE6, lwd=2, col = 4 )
abline(h = est.CAPE6, col = 1);
abline(h = ubound.CAPE6, col = 1, lty = 2);
abline(h = lbound.CAPE6, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

dev.off();



cairo_pdf("Gender-CSPET-IN-bc-qr-full.pdf", pointsize=15,width=12.0,height=8.0);

par(mfrow=c(2,3));

plot(us, est.CSPE1.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Manufacture",
     sub="Quantile model", 
     ylim=c(-.7,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1.bc,rev(lbound.CSPE1.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1.bc, lwd=2, col = 4 )
abline(h = est.CAPE1.bc, col = 1);
abline(h = ubound.CAPE1.bc, col = 1, lty = 2);
abline(h = lbound.CAPE1.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Finance",
     sub="Quantile model", 
     ylim=c(-.7,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2.bc,rev(lbound.CSPE2.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2.bc, lwd=2, col = 4 )
abline(h = est.CAPE2.bc, col = 1);
abline(h = ubound.CAPE2.bc, col = 1, lty = 2);
abline(h = lbound.CAPE2.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Professional",
     sub="Quantile model",
     ylim=c(-.7,.4))
polygon(c(us,rev(us)),c(ubound.CSPE3.bc,rev(lbound.CSPE3.bc)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3.bc, lwd=2, col = 4 )
abline(h = est.CAPE3.bc, col = 1);
abline(h = ubound.CAPE3.bc, col = 1, lty = 2);
abline(h = lbound.CAPE3.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE4.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Education",
     sub="Quantile model",
     ylim=c(-.7,.4))
polygon(c(us,rev(us)),c(ubound.CSPE4.bc,rev(lbound.CSPE4.bc)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE4.bc, lwd=2, col = 4 )
abline(h = est.CAPE4.bc, col = 1);
abline(h = ubound.CAPE4.bc, col = 1, lty = 2);
abline(h = lbound.CAPE4.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE5.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Leisure",
     sub="Quantile model",
     ylim=c(-.7,.4))
polygon(c(us,rev(us)),c(ubound.CSPE5.bc,rev(lbound.CSPE5.bc)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE5.bc, lwd=2, col = 4 )
abline(h = est.CAPE5.bc, col = 1);
abline(h = ubound.CAPE5.bc, col = 1, lty = 2);
abline(h = lbound.CAPE5.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE6.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap",
     main="Public",
     sub="Quantile model",
     ylim=c(-.7,.4))
polygon(c(us,rev(us)),c(ubound.CSPE6.bc,rev(lbound.CSPE6.bc)), density=60, border=F,
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE6.bc, lwd=2, col = 4 )
abline(h = est.CAPE6.bc, col = 1);
abline(h = ubound.CAPE6.bc, col = 1, lty = 2);
abline(h = lbound.CAPE6.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

dev.off();



###################################################################################;
# Part 7: Conditional effects by ranking in conditional distribution for women;
###################################################################################;


########## statistics to be computed in each bootstrap draw #####################################
boot.PE<- function(data,indices, fmla1, taus, us){
  #  data.b<- data[indices,];  
  n<-dim(data)[1]
  #  qr.fit <- rq.nostop(fmla1, data=data, weights = data$weight*rexp(n), tau = taus, method = "pfn");
  multipliers  <- rexp(n);
  data$bweight <-data$weight*multipliers/sum(multipliers);
  rm(multipliers);
  qr.fit <- rq.nostop(fmla1, data=data, weights = bweight, tau = taus, method = "sfn2",  control = list(tmpmax = 1e6));
  ## design matrices for PE
  d0<- qr.fit$model;  
  d0[,2]<- rep(0,n);
  d1<- qr.fit$model;  
  d1[,2]<- rep(1,n);
  # PEs
  PE    <- predict(qr.fit, newdata=d1) -  predict(qr.fit, newdata=d0);
  rm(d1,d0);  
  PE.lq <- PE[data$female==1, 1];
  w.lq  <- data$bweight[data$female==1];  
  PE.mq <- PE[data$female==1, 2];
  w.mq  <- data$bweight[data$female==1];    
  PE.hq <- PE[data$female==1, 3];
  w.hq  <- data$bweight[data$female==1];  
  if (sum(is.na(PE.lq)) == 0) {
    return.lq <- c(weighted.mean(PE.lq, w.lq), wtd.quantile(PE.lq, w.lq, us)) 
  } else {
    return.lq <- rep(NA, length(us) + 1);
  }
  
  if (sum(is.na(PE.mq)) == 0) {
    return.mq <- c(weighted.mean(PE.mq, w.mq), wtd.quantile(PE.mq, w.mq, us)) 
  } else {
    return.mq <- rep(NA, length(us) + 1);
  }
  
  if (sum(is.na(PE.hq)) == 0) {
    return.hq <- c(weighted.mean(PE.hq, w.hq), wtd.quantile(PE.hq, w.hq, us)) 
  } else {
    return.hq <- rep(NA, length(us) + 1);
  }
  return(c(return.lq, return.mq, return.hq));
  
}

taus2 <- c(1,5,9)/10;
set.seed(1);
result.boot.PE.qr<- boot(data=data, statistic=boot.PE, fmla1 = fmla1, taus = taus2, us = us, parallel="multicore", ncpus = 15, R=R); 

# CSPEs;

qr.fit2     <- rq.nostop(fmla1, weights = weight, tau = taus2, method = "sfn2",  control = list(tmpmax = 1e6)); 
d0<- qr.fit2$model;  
d0[,2]<- rep(0,n);
d1<- qr.fit2$model;  
d1[,2]<- rep(1,n);

PE.qr       <- predict(qr.fit2, newdata=d1) -  predict(qr.fit2, newdata=d0);
rm(d0,d1);

# First decile;

PE.lq <- PE.qr[female==1, 1];
w.lq  <- weight[female==1];
est.CSPE1<- wtd.quantile(PE.lq, w.lq, us);   # these are estimated SPEs
draws.CSPE1<- result.boot.PE.qr$t[,1 + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta     <- draws.CSPE1-matrix(est.CSPE1, nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE);
variance  <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs        <- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE1), ncol = ncol(draws.CSPE1), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt       <- quantile(zs, 1-alpha);  #critical value

ubound.CSPE1<-  sort(est.CSPE1 + crt*sqrt(variance));  #upper conf band
lbound.CSPE1<-  sort(est.CSPE1 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE1.bc <- sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean));
ubound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE1.bc <-  sort(2*est.CSPE1 - apply(draws.CSPE1,2,mean) - crt*sqrt(variance));  #lower conf band


# Median;

PE.mq <- PE.qr[female==1, 2];
w.mq  <- weight[female==1];
est.CSPE2<- wtd.quantile(PE.mq, w.mq, us);   # these are estimated SPEs
draws.CSPE2<- result.boot.PE.qr$t[,2 + length(us) + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE2-matrix(est.CSPE2, nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE2), ncol = ncol(draws.CSPE2), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE2<-  sort(est.CSPE2 + crt*sqrt(variance));  #upper conf band
lbound.CSPE2<-  sort(est.CSPE2 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE2.bc <- sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean));
ubound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE2.bc <-  sort(2*est.CSPE2 - apply(draws.CSPE2,2,mean) - crt*sqrt(variance));  #lower conf band


# Ninth Decile;

PE.hq <- PE.qr[female==1, 3];
w.hq  <- weight[female==1];
est.CSPE3<- wtd.quantile(PE.hq, w.hq, us);   # these are estimated SPEs
draws.CSPE3<- result.boot.PE.qr$t[,3 + 2*length(us) + 1:length(us)];  #these are bs draws

#centered/scaled draws
delta    <- draws.CSPE3-matrix(est.CSPE3, nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm = TRUE) - apply(delta, 2, quantile, .25, na.rm = TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.CSPE3), ncol = ncol(draws.CSPE3), byrow=TRUE)), 1, max, na.rm = TRUE);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.CSPE3<-  sort(est.CSPE3 + crt*sqrt(variance));  #upper conf band
lbound.CSPE3<-  sort(est.CSPE3 - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.CSPE3.bc <- sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean));
ubound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) + crt*sqrt(variance));  #upper conf band
lbound.CSPE3.bc <-  sort(2*est.CSPE3 - apply(draws.CSPE3,2,mean) - crt*sqrt(variance));  #lower conf band


# APEs;

# First Decile;

est.CAPE1  <- weighted.mean(PE.lq, w.lq);
draws.CAPE1<- result.boot.PE.qr$t[,1];
delta    <- draws.CAPE1 - est.CAPE1;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt1     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE1<-  sort(est.CAPE1+ crt1*sqrt(variance));  #upper conf bound
lbound.CAPE1<-  sort(est.CAPE1 - crt1*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE1.bc  <- 2*est.CAPE1 - mean(draws.CAPE1);
ubound.CAPE1.bc <- est.CAPE1.bc + crt1*sqrt(variance);  #upper conf bound
lbound.CAPE1.bc <- est.CAPE1.bc - crt1*sqrt(variance);  #lower conf bound


# Median;


est.CAPE2  <- weighted.mean(PE.mq, w.mq);
draws.CAPE2<- result.boot.PE.qr$t[,2 + length(us)];
delta    <- draws.CAPE2 - est.CAPE2;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt2     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE2<-  sort(est.CAPE2+ crt2*sqrt(variance));  #upper conf bound
lbound.CAPE2<-  sort(est.CAPE2 - crt2*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE2.bc  <- 2*est.CAPE2 - mean(draws.CAPE2);
ubound.CAPE2.bc <- est.CAPE2.bc + crt2*sqrt(variance);  #upper conf bound
lbound.CAPE2.bc <- est.CAPE2.bc - crt2*sqrt(variance);  #lower conf bound


# Ninth Decile;

est.CAPE3  <- weighted.mean(PE.hq, w.hq);
draws.CAPE3<- result.boot.PE.qr$t[,3 + 2*length(us)];
delta    <- draws.CAPE3 - est.CAPE3;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
mzs      <- abs(delta /sqrt(variance));  
crt3     <- quantile(mzs, 1-alpha, na.rm = TRUE);  #critical value

ubound.CAPE3<-  sort(est.CAPE3+ crt3*sqrt(variance));  #upper conf bound
lbound.CAPE3<-  sort(est.CAPE3 - crt3*sqrt(variance));  #lower conf bound

# bootstrap bias correction;

est.CAPE3.bc  <- 2*est.CAPE3 - mean(draws.CAPE3);
ubound.CAPE3.bc <- est.CAPE3.bc + crt3*sqrt(variance);  #upper conf bound
lbound.CAPE3.bc <- est.CAPE3.bc - crt3*sqrt(variance);  #lower conf bound


# Conditional Expectation:

########## statistics to be computed in each bootstrap draw #####################################
boot.PE.lm<- function(data,indices, fmla1, us){
  #  data.b<- data[indices,];  
  #basic linear model with interactions
  n<-dim(data)[1];
  multipliers  <- rexp(n);
  data$bweight <-data$weight*multipliers/sum(multipliers);
  rm(multipliers);
  lm.fit <- lm(fmla1, data=data, weights = bweight);
  ## design matrices for PE
  d0<- lm.fit$model;  
  d0[,2]<- rep(0,n);
  d1<- lm.fit$model;  
  d1[,2]<- rep(1,n);
  PE.lm   <- predict(lm.fit, newdata=d1) -  predict(lm.fit, newdata=d0);
  PET.lm  <- PE.lm[data$female==1];
  PET.weight <- data$bweight[data$female==1];
  return(c(weighted.mean(PE.lm, data$bweight), wtd.quantile(PE.lm, data$bweight, us), weighted.mean(PET.lm, PET.weight), wtd.quantile(PET.lm, PET.weight, us)));
}

set.seed(1);
result.boot.PE.lm<- boot(data=data, statistic=boot.PE.lm, fmla1 = fmla1, us = us, parallel="multicore", ncpus = 20, R=500);

#fmla1.lm   <- lnw ~ female* (widowed + divorced + separated + nevermarried + (exp1+exp2+exp3+exp4)*(educ+occ2+ind2) + (educ+occ2+ind2)^2 + mw + so + we);
#lm.fit     <- lm(fmla1.lm, weights = weight);
lm.fit     <- lm(fmla1, weights = weight);

## desing matrix
d0<- lm.fit$model;  
d0[,2]<- rep(0,n);
d1<- lm.fit$model;  
d1[,2]<- rep(1,n);

# SPE-T;
est.SPET<- wtd.quantile(predict(lm.fit, newdata=d1)[female==1] -   predict(lm.fit, newdata=d0)[female==1], weight[female==1],  us);   # these are estimated SPEs
draws.SPET<- result.boot.PE.lm$t[,-c(1:(length(us)+2))];  #these are bs draws

# centered/scaled draws

delta    <- draws.SPET-matrix(est.SPET, nrow = nrow(draws.SPET), ncol = ncol(draws.SPET), byrow=TRUE);
variance      <- ((apply(delta, 2, quantile, .75, na.rm=TRUE) - apply(delta, 2, quantile, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator
zs<- apply(abs(delta /matrix(sqrt(variance), nrow = nrow(draws.SPET), ncol = ncol(draws.SPET), byrow=TRUE)), 1, max);  #max abs t-stat
crt<- quantile(zs, 1-alpha);  #critical value

ubound.SPET<-  sort(est.SPET + crt*sqrt(variance));  #upper conf band
lbound.SPET<-  sort(est.SPET - crt*sqrt(variance));  #lower conf band

# bootstrap bias correction;

est.SPET.bc  <- sort(2*est.SPET - apply(draws.SPET,2,mean));
ubound.SPET.bc <- sort(2*est.SPET - apply(draws.SPET,2,mean) + crt*sqrt(variance));  #upper conf bound
lbound.SPET.bc <- sort(2*est.SPET - apply(draws.SPET,2,mean) - crt*sqrt(variance));  #lower conf bound


# APE-T;
est.APET  <- weighted.mean(predict(lm.fit, newdata=d1)[female==1] -   predict(lm.fit, newdata=d0)[female==1], weight[female==1]);
draws.APET<- result.boot.PE.lm$t[,2+length(us)];
delta    <- draws.APET - est.APET;
variance      <- ((quantile(delta, .75, na.rm=TRUE) - quantile(delta, .25, na.rm=TRUE))/(qnorm(0.75) - qnorm(.25)))^2; # Robust variance estimator 
mzs      <- abs(delta /sqrt(variance));  
crt2     <- quantile(mzs, 1-alpha);  #critical value

ubound.APET<- est.APET + crt2*sqrt(variance);  #upper conf bound
lbound.APET<- est.APET - crt2*sqrt(variance);  #lower conf bound

# bootstrap bias correction;

est.APET.bc  <- 2*est.APET - mean(draws.APET);
ubound.APET.bc <- est.APET.bc + crt2*sqrt(variance);  #upper conf bound
lbound.APET.bc <- est.APET.bc - crt2*sqrt(variance);  #lower conf bound


# Figure of results;

cairo_pdf("Gender-CSPET-RANK-qr.pdf", pointsize=15,width=8.0,height=4.0);

par(mfrow=c(1,3));

plot(us, est.CSPE1, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="First Decile",
     sub="Quantile model", 
     ylim=c(-1,.5))
polygon(c(us,rev(us)),c(ubound.CSPE1,rev(lbound.CSPE1)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1, lwd=2, col = 4 )
abline(h = est.CAPE1, col = 1);
abline(h = ubound.CAPE1, col = 1, lty = 2);
abline(h = lbound.CAPE1, col = 1, lty = 2);
legend(0, .5, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .5, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Median",
     sub="Quantile model", 
     ylim=c(-1,.5))
polygon(c(us,rev(us)),c(ubound.CSPE2,rev(lbound.CSPE2)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2, lwd=2, col = 4 )
abline(h = est.CAPE2, col = 1);
abline(h = ubound.CAPE2, col = 1, lty = 2);
abline(h = lbound.CAPE2, col = 1, lty = 2);
legend(0, .5, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .5, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Ninth Decile",
     sub="Quantile model", 
     ylim=c(-1,.5))
polygon(c(us,rev(us)),c(ubound.CSPE3,rev(lbound.CSPE3)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3, lwd=2, col = 4 )
abline(h = est.CAPE3, col = 1);
abline(h = ubound.CAPE3, col = 1, lty = 2);
abline(h = lbound.CAPE3, col = 1, lty = 2);
legend(0, .5, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .5, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();


cairo_pdf("Gender-CSPET-RANK-bc-qr.pdf", pointsize=15,width=8.0,height=4.0);

par(mfrow=c(1,3));

plot(us, est.CSPE1.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="First Decile",
     sub="Quantile model", 
     ylim=c(-.9,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1.bc,rev(lbound.CSPE1.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1.bc, lwd=2, col = 4 )
abline(h = est.CAPE1.bc, col = 1);
abline(h = ubound.CAPE1.bc, col = 1, lty = 2);
abline(h = lbound.CAPE1.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');

plot(us, est.CSPE2.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Median",
     sub="Quantile model", 
     ylim=c(-.9,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2.bc,rev(lbound.CSPE2.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2.bc, lwd=2, col = 4 )
abline(h = est.CAPE2.bc, col = 1);
abline(h = ubound.CAPE2.bc, col = 1, lty = 2);
abline(h = lbound.CAPE2.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Ninth Decile",
     sub="Quantile model", 
     ylim=c(-.9,.4))
polygon(c(us,rev(us)),c(ubound.CSPE3.bc,rev(lbound.CSPE3.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3.bc, lwd=2, col = 4 )
abline(h = est.CAPE3.bc, col = 1);
abline(h = ubound.CAPE3.bc, col = 1, lty = 2);
abline(h = lbound.CAPE3.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();



cairo_pdf("Gender-CSPET-RANK-MEAN-qr.pdf", pointsize=15,width=8.0,height=8.0);

par(mfrow=c(2,2));

plot(us, est.SPET, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Mean",
     sub="Mean model", 
     ylim=c(-1,.5))
polygon(c(us,rev(us)),c(ubound.SPET,rev(lbound.SPET)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.SPET, lwd=2, col = 4 )
abline(h = est.APET, col = 1);
abline(h = ubound.APET, col = 1, lty = 2);
abline(h = lbound.APET, col = 1, lty = 2);
legend(0, .5, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .5, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE2, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Median",
     sub="Quantile model", 
     ylim=c(-1,.5))
polygon(c(us,rev(us)),c(ubound.CSPE2,rev(lbound.CSPE2)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2, lwd=2, col = 4 )
abline(h = est.CAPE2, col = 1);
abline(h = ubound.CAPE2, col = 1, lty = 2);
abline(h = lbound.CAPE2, col = 1, lty = 2);
legend(0, .5, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .5, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE1, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="First Decile",
     sub="Quantile model", 
     ylim=c(-1,.5))
polygon(c(us,rev(us)),c(ubound.CSPE1,rev(lbound.CSPE1)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1, lwd=2, col = 4 )
abline(h = est.CAPE1, col = 1);
abline(h = ubound.CAPE1, col = 1, lty = 2);
abline(h = lbound.CAPE1, col = 1, lty = 2);
legend(0, .5, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .5, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Ninth Decile",
     sub="Quantile model", 
     ylim=c(-1,.5))
polygon(c(us,rev(us)),c(ubound.CSPE3,rev(lbound.CSPE3)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3, lwd=2, col = 4 )
abline(h = est.CAPE3, col = 1);
abline(h = ubound.CAPE3, col = 1, lty = 2);
abline(h = lbound.CAPE3, col = 1, lty = 2);
legend(0, .5, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .5, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();

cairo_pdf("Gender-CSPET-RANK-MEAN-bc-qr.pdf", pointsize=15,width=8.0,height=8.0);

par(mfrow=c(2,2));

plot(us, est.SPET.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Mean",
     sub="Mean model", 
     ylim=c(-.9,.4))
polygon(c(us,rev(us)),c(ubound.SPET.bc,rev(lbound.SPET.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.SPET.bc, lwd=2, col = 4 )
abline(h = est.APET.bc, col = 1);
abline(h = ubound.APET.bc, col = 1, lty = 2);
abline(h = lbound.APET.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE2.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Median",
     sub="Quantile model", 
     ylim=c(-.9,.4))
polygon(c(us,rev(us)),c(ubound.CSPE2.bc,rev(lbound.CSPE2.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE2.bc, lwd=2, col = 4 )
abline(h = est.CAPE2.bc, col = 1);
abline(h = ubound.CAPE2.bc, col = 1, lty = 2);
abline(h = lbound.CAPE2.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE1.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="First Decile",
     sub="Quantile model", 
     ylim=c(-.9,.4))
polygon(c(us,rev(us)),c(ubound.CSPE1.bc,rev(lbound.CSPE1.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE1.bc, lwd=2, col = 4 )
abline(h = est.CAPE1.bc, col = 1);
abline(h = ubound.CAPE1.bc, col = 1, lty = 2);
abline(h = lbound.CAPE1.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


plot(us, est.CSPE3.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     main="Ninth Decile",
     sub="Quantile model", 
     ylim=c(-.9,.4))
polygon(c(us,rev(us)),c(ubound.CSPE3.bc,rev(lbound.CSPE3.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.CSPE3.bc, lwd=2, col = 4 )
abline(h = est.CAPE3.bc, col = 1);
abline(h = ubound.CAPE3.bc, col = 1, lty = 2);
abline(h = lbound.CAPE3.bc, col = 1, lty = 2);
legend(0, .4, c(' ', ' ' ), col = c('light blue',1), lwd = c(4,1), horiz = F, bty = 'n');
legend(0, .4, c('SPE','APE'), col = c(4,1), lwd = c(1,1), horiz = F, bty = 'n');


dev.off();

pdf("Gender-SPET-bc-lm.pdf", pointsize=15,width=8.0,height=8.0)
plot(us, est.SPET.bc, type="l",col=4, xlab="Percentile index", lwd=2,
     ylab="Gender wage gap", 
     #     main="Women",
     sub="Mean model", 
     ylim=c(-.65, .25))

polygon(c(us,rev(us)),c(ubound.SPET.bc,rev(lbound.SPET.bc)), density=60, border=F, 
        col='light blue', lty = 1, lwd = 1);
lines(us, est.SPET.bc, lwd=2, col = 4 )
abline(h = est.APET.bc, col = 1);
abline(h = ubound.APET.bc, col = 1, lty = 2);
abline(h = lbound.APET.bc, col = 1, lty = 2);
points(c(min(us),.2,.4,.6,.8,max(us)), rep(est.APET.bc,6),  col=1, pch=15 )
legend( x="topleft", 
        legend=c("SPE","APE","90% CB(SPE)", "90% CB(APE)"), 
        col=c(4,1,"light blue",1), lwd=c(1, 1, 5, 1), lty=c(1, 1, 1, 2), pch=c(NA, 15, NA, NA), pt.cex = c(2, 1), bty = 'n' )

dev.off();

# 3D figure for PE with respect to ranking and quantile;

library(lattice); #load library to make nice 3d plots
 

SPET      <- matrix(t(apply(PE[which(female==1),], 2, wtd.quantile, weight[female==1], us)), ncol = 1);
coors     <- expand.grid(taus,us);
colnames(coors) <- c('ranking','quantile');

trellis.device(pdf, file = "Gender-CSPET-RANK-3d-qr.pdf", color = TRUE, height=20, width=20, pointsize=10)

trellis.par.set(list(axis.text=list(cex=2),theme = col.whitebg()));

trellis.par.set("axis.line", list(col="transparent"));
plot3d3 <- wireframe(SPET ~ coors$ranking * coors$quantile,  ylab = expression(scriptstyle("Quantile")), xlab = expression(scriptstyle("Ranking")), zlab = expression(scriptstyle("Wage Gap")),  zlim = range(SPET), 
                     lwd = .5, cex.lab = 2, scales = list(arrows = FALSE, tick.number = 4, col = 1, cex = 1), drape = T, col.regions = "lightblue", colorkey=F, main = " ");
print(plot3d3);

dev.off()



save.image(file="Gender-qr-full-trim10-june2018.RData");

detach(data);

